#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2023
my $ver="2.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"f=s","b=s","o=s","s=s","h");

#&help()if(defined $opts{h});
if(!defined($opts{f}) || !defined($opts{b}) || !defined($opts{s}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver
		   v1.5: flu file change to 18-cols.
		   v1.6: add barcode.tsv outfile.

	Usage:

		-f           flu data file                     <infile>     must be given
		-b           barcode index file                <infile>     must be given
		-s           seq barcode file                  <infile>     must be given
		-o           result file                       <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $bcindexfile = $opts{b} ;
my $flufile = $opts{f} ;
my $bcumifile = $opts{s} ;
my $outfile = $opts{o} ;

## reading barcode index file
my %hindex = ();
my %hindex2flu = ();
&show_log("start get barcode index");
&get_barcode_index($bcindexfile, \%hindex, \%hindex2flu);

## count barcode diff
my %hbc_diff = ();
my %hdiff2bc = ();
&count_bc_diff(\%hindex, \%hbc_diff, \%hdiff2bc);

## reading seq barcode file
my %hseqbc = ();
my %hlink1 = ();
&show_log("start reading seq barcode");
&get_bc_umi_info($bcumifile, \%hseqbc, \%hlink1);

## reading flufile
my %hdup = ();
my @flu_pos = ();
my %hconf_bc = ();
&show_log("start get flu info1");
&get_conf_bc($flufile, \%hindex, \%hseqbc, \%hdup, \@flu_pos, \%hconf_bc);
my %hbc_stat = ();
&show_log("start get flu info2");
&get_flu_info($flufile, \%hindex, \%hseqbc, \%hlink1, \%hdup, \@flu_pos, \%hconf_bc, \%hbc_stat);

## get flu and bc relation
&show_log("start get flu and bc relation");
my @final_flu_pos = ();
&get_relation(\@flu_pos, \%hdup, \%hseqbc, \%hconf_bc, \%hbc_stat, $outfile, \@final_flu_pos);

## out put
&show_log("start output");
&out_put(\@final_flu_pos, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

#&get_barcode_index($bcindexfile, \%hindex, \%hindex2flu);
sub get_barcode_index()
{
    my ($bcindexfile, $ahindex, $ahindex2flu) = @_ ;

    open (IN, $bcindexfile) || die "$bcindexfile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/Index|^\s*$/);
        my ($index, $seq, @arr) = split ;
        my $str ;
        my $value ;
        for (my $i=0; $i<6; $i++){
            my $cy3 = $arr[$i] ;
            my $cy5 = $arr[$i+6] ;
            if ($cy3 == 1){
                $value = 0 ;
            }
            elsif ($cy5 == 1){
                $value = 1 ;
            }
            else{
                $value = 2 ;
            }
            if ($cy3 && $cy5){
                print "error: $_\n" ;
                exit(1);
            }
            $str .= $value ;
        }
        $ahindex->{$str} = $index ;
        $ahindex2flu->{$index} = $str ;
    }
    close(IN);

    return ;
}

#&count_bc_diff(\%hindex, \%hbc_diff, \%hdiff2bc);
sub count_bc_diff()
{
    my ($ahindex, $ahbc_diff, $ahdiff2bc) = @_ ;

    my @strs = keys %{$ahindex} ;
    for (my $i=0; $i<@strs-1; $i++){
        my $s1 = $strs[$i] ;
        my $index1 = $ahindex->{$s1} ;
        for (my $j=$i+1; $j<@strs; $j++){
            my $s2 = $strs[$j] ;
            my $index2 = $ahindex->{$s2} ;
            my $diff = &count_diff($s1, $s2);
            $ahbc_diff->{$s1}{$s2} = $diff ;
            $ahbc_diff->{$s2}{$s1} = $diff ;
            push @{$ahdiff2bc->{$diff}{$index1}}, $index2 ;
            push @{$ahdiff2bc->{$diff}{$index2}}, $index1 ;
        }
    }

    return ;
}

#my $diff = &count_diff($s1, $s2);
sub count_diff()
{
    my ($s1, $s2) = @_ ;

    my $mask = $s1 ^ $s2 ;
    my $diff = () = $mask =~ m/[^\0]/g ;

    return($diff) ;
}

#&get_bc_umi_info($bcumifile, \%hseqbc, \%hlink1);
sub get_bc_umi_info()
{
    my ($bcumifile, $ahbc, $ahlink1) = @_ ;

    open (IN, $bcumifile) || die "$bcumifile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($bc, $umi) = split ;
        $ahbc->{$bc}++ ;
        my ($bc1, $bc2, $bc3) = split /\-/, $bc ;
        $ahlink1->{$bc1}{$bc2} = 1 ;
    }
    close(IN);

    return ;
}

#&get_conf_bc($flufile, \%hindex, \%hseqbc, \%hdup, \@flu_pos, \%hconf_bc);
sub get_conf_bc()
{
    my ($flufile, $ahindex, $ahseqbc, $ahdup, $aflu_pos, $ahconf_bc) = @_ ;

    open (IN, $flufile) || die "$flufile, $!\n" ;
    my $index = 0 ;
    my %hflu = ();
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        $index++ ;
        my $pos = &change2pos($index);
        my ($str) = split ;
        if (defined $hflu{$str}){
            $ahdup->{$index} = $pos ;
            $ahdup->{$hflu{$str}} = &change2pos($hflu{$str}) ;
            delete $aflu_pos->[$hflu{$str}-1] ;
            next ;
        }
        else{
            $hflu{$str} = $index ;
        }
        if ($str =~ m/(\d{6})(\d{6})(\d{6})/){
            my ($str1, $str2, $str3) = ($1, $2, $3) ;
            if (defined $ahindex->{$str1} && defined $ahindex->{$str2} && defined $ahindex->{$str3}){
                my $bc_type = "bc1_".$ahindex->{$str1}."-bc2_".$ahindex->{$str2}."-bc3_".$ahindex->{$str3} ;
                if (defined $ahseqbc->{$bc_type}){
                    $ahconf_bc->{$bc_type} = 1 ;
                    $aflu_pos->[$index-1] = [$pos, 0, $bc_type] ;
                }
            }
        }
    }
    close(IN);

    return ;
}

#&get_flu_info($flufile, \%hindex, \%hseqbc, \%hlink1, \%hdup, \@flu_pos, \%hconf_bc, \%hbc_stat);
sub get_flu_info()
{
    my ($flufile, $ahindex, $ahseqbc, $ahlink1, $ahdup, $aflu_pos, $ahconf_bc, $ahbc_stat) = @_ ;

    open (IN, $flufile) || die "$flufile, $!\n" ;
    my $index = 0 ;
    my %hstr2bc = ();
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        $index++ ;
        next if (defined $ahdup->{$index}) ;
        next if (defined $aflu_pos->[$index-1]) ;
        my $pos = &change2pos($index);
        my ($str) = split ;
        if ($str =~ m/(\d{6})(\d{6})(\d{6})/){
            my ($str1, $str2, $str3) = ($1, $2, $3) ;
            my ($flag, $diff, @bc_types) = &judge_bc_type($str1, $str2, $str3, $ahindex, $ahseqbc, $ahlink1, $ahconf_bc, \%hstr2bc);
            if ($flag == 1){
                next ;
            }
            $aflu_pos->[$index-1] = [$pos, $diff, @bc_types] ;
            for my $bc (@bc_types){
                $ahbc_stat->{$bc}++ ;
            }
        }
    }
    close(IN);

    return ;
}

#my ($flag, $diff, @bc_types) = &judge_bc_type($str1, $str2, $str3, $ahindex, $ahseqbc, $ahlink1, $ahconf_bc, \%hstr2bc);
sub judge_bc_type()
{
    my ($s1, $s2, $s3, $ahindex, $ahseqbc, $ahlink1, $ahconf_bc, $ahstr2bc) = @_ ;

    my $flag = 0 ;
    my @bc_strs = ($s1, $s2, $s3);
    my @candi_bcs = ();
    my $diff = 0 ;
    for (my $i=0; $i<@bc_strs; $i++){
        my $str = $bc_strs[$i] ;
        if (defined $ahindex->{$str}){
            $candi_bcs[$i][0] = "bc".($i+1)."_".$ahindex->{$str} ;
        }
        else{
            if (defined $ahstr2bc->{$str}){
                @{$candi_bcs[$i]} = @{$ahstr2bc->{$str}} ;
                return 1 if (scalar @{$candi_bcs[$i]} == 0) ;
            }
            else{
                (my $candi_flag, @{$candi_bcs[$i]}) = &get_candi_bcs($str, $ahindex, $ahstr2bc, "bc".($i+1));
                return 1 if ($candi_flag == 0);
            }
            $diff += 1 ;
        }
    }
    my ($select_flag, @bc_types) = &get_bc_type(\@candi_bcs, $ahseqbc, $ahlink1, $ahconf_bc);
    if ($select_flag == 0){
        ($select_flag, $diff, @bc_types) = &get_bc_type_loose($s1, $s2, $s3, $ahindex, $ahstr2bc, $ahseqbc, $ahlink1, $ahconf_bc);
    }

    return($flag, $diff, @bc_types);
}

#($select_flag, $diff, @bc_types) = &get_bc_type_loose($s1, $s2, $s3, $ahindex, $ahstr2bc, $ahseqbc, $ahlink1, $ahconf_bc);
sub get_bc_type_loose()
{
    my ($s1, $s2, $s3, $ahindex, $ahstr2bc, $ahseqbc, $ahlink1, $ahconf_bc) = @_ ;

    my @bc_strs = ($s1, $s2, $s3);
    my @candi_bcs = ();
    my %hdiff = ();
    for (my $i=0; $i<@bc_strs; $i++){
        my $str = $bc_strs[$i] ;
        if (defined $ahindex->{$str}){
            $candi_bcs[$i][0] = "bc".($i+1)."_".$ahindex->{$str} ;
            $hdiff{$candi_bcs[$i][0]} = 0 ;
        }
        if (defined $ahstr2bc->{$str}){
            push @{$candi_bcs[$i]}, @{$ahstr2bc->{$str}} ;
            for my $bc (@{$ahstr2bc->{$str}}){
                $hdiff{$bc} = 1 ;
            }
        }
        else{
            my ($candi_flag, @bcs) = &get_candi_bcs($str, $ahindex, $ahstr2bc, "bc".($i+1));
            push @{$candi_bcs[$i]}, @bcs ;
            for my $bc (@bcs){
                $hdiff{$bc} = 1 ;
            }
        }
    }
    my ($select_flag, $diff, @bc_types) = &get_candi_bc_types(\@candi_bcs, $ahseqbc, $ahlink1, $ahconf_bc, \%hdiff);

    return($select_flag, $diff, @bc_types);
}

#my ($select_flag, $diff, @bc_types) = &get_candi_bc_types(\@candi_bcs, $ahseqbc, $ahlink1, $ahconf_bc, \%hdiff);
sub get_candi_bc_types()
{
    my ($abcs, $ahseqbc, $ahlink1, $ahconf_bc, $ahdiff) = @_ ;

    my $flag = 0 ;
    my %hbcs = ();
    for my $bc1 (@{$abcs->[0]}){
        for my $bc2 (@{$abcs->[1]}){
            next if (!defined $ahlink1->{$bc1}{$bc2});
            for my $bc3 (@{$abcs->[2]}){
                next if (defined $ahconf_bc->{"$bc1-$bc2-$bc3"}) ;
                if (defined $ahseqbc->{"$bc1-$bc2-$bc3"}){
                    my $diff = $ahdiff->{$bc1} + $ahdiff->{$bc2} + $ahdiff->{$bc3} ;
                    $hbcs{"$bc1-$bc2-$bc3"} = $diff ;
                    $flag = 1 ;
                }
            }
        }
    }
    return (0) if ($flag == 0);
    my @candi_bcs = sort {$hbcs{$a} <=> $hbcs{$b}} keys %hbcs ;
    my @bc_types = ($candi_bcs[0]);
    for (my $i=1; $i<@candi_bcs; $i++){
        if ($hbcs{$candi_bcs[$i]} == $hbcs{$candi_bcs[$i-1]}){
            push @bc_types, $candi_bcs[$i] 
        }
        else{
            last ;
        }
    }
    my $diff = $hbcs{$bc_types[0]} ;

    return($flag, $diff, @bc_types);
}

#my ($select_flag, @bc_types) = &get_bc_type(\@candi_bcs, $ahseqbc, $ahlink1, $ahconf_bc);
sub get_bc_type()
{
    my ($abcs, $ahseqbc, $ahlink1, $ahconf_bc) = @_ ;

    my $flag = 0 ;
    my @bc_types = ();
    for my $bc1 (@{$abcs->[0]}){
        for my $bc2 (@{$abcs->[1]}){
            next if (!defined $ahlink1->{$bc1}{$bc2});
            for my $bc3 (@{$abcs->[2]}){
                next if (defined $ahconf_bc->{"$bc1-$bc2-$bc3"}) ;
                if (defined $ahseqbc->{"$bc1-$bc2-$bc3"}){
                    push @bc_types, "$bc1-$bc2-$bc3" ;
                    $flag = 1 ;
                }
            }
        }
    }

    return($flag, @bc_types);
}

#@bc1s = &get_candi_bcs($s1, $ahindex, $ahstr2bc, "bc1");
sub get_candi_bcs()
{
    my ($str, $ahindex, $ahstr2bc, $type) = @_ ;

    my @bcs = ();
    my $flag = 0 ;
    for my $str2 (keys %{$ahindex}){
        my $mask = $str ^ $str2 ;
        my $num = () = $mask =~ m/[^\0]/g ;
        my $index = $ahindex->{$str2} ;
        if ($num == 1){
            push @bcs, "$type\_$index" ;
            $flag = 1 ;
        }
    }
    $ahstr2bc->{$str} = \@bcs ;

    return($flag, @bcs);
}

#my $pos = &change2pos($index);
sub change2pos()
{
    my ($index) = @_ ;

    $index -= 1 ;

    my $block = int($index/30/35) ;
    my $block_row = int($block/46) ;
    my $block_col = $block % 46 ;

    my $pos = $index % (30*35);
    my $pos_row = int($pos/30) ;
    my $pos_col = $pos % 30 ;

    return("$block_row\t$block_col\t$pos_row\t$pos_col");
}

#&get_relation(\@flu_pos, \%hdup, \%hseqbc, \%hconf_bc, \%hbc_stat, $outfile, \@final_flu_pos);
sub get_relation()
{
    my ($aflu_pos, $ahdup, $ahseqbc, $ahconf_bc, $ahbc_stat, $outfile, $afinal_flu_pos) = @_ ;

    open (OUT, ">$outfile.info") || die "$outfile.info, $!\n" ;
    my @final_pos_bc = ();
    my %hstat = ();
    my %hcandi = ();
    my @candi_pos_bc = ();
    for (my $i=0; $i<@{$aflu_pos}; $i++){
        if (defined $ahdup->{$i+1}){
            print OUT $i+1,"\t",$ahdup->{$i+1}, "\tDupFlu\tNull\n" ;
            $hstat{Dup}++ ;
        }
        elsif (!defined $aflu_pos->[$i]){
            print OUT $i+1,"\t",&change2pos($i+1), "\tErrFlu\tNull\n" ;
            $hstat{Error}++ ;
        }
        else{
            my ($pos, $diff, @bc_types) = @{$aflu_pos->[$i]} ;
            print OUT $i+1,"\t", $pos ;
            if (scalar(@bc_types) == 1){
                if (defined $ahconf_bc->{$bc_types[0]}){
                    print OUT "\tUniq\t", $bc_types[0],"\n" ;
                    $final_pos_bc[$i] = [$pos, $bc_types[0]] ;
                    $hstat{Uniq}++ ;
                }
                else{
                    if ($ahbc_stat->{$bc_types[0]} == 1){
                        print OUT "\tUniq\t", $bc_types[0],"\n" ;
                        $final_pos_bc[$i] = [$pos, $bc_types[0]] ;
                        $hstat{"E-Uniq"}++ ;
                    }
                    else{
                        print OUT "\tMultPos\t$bc_types[0]\n" ;
                        $hcandi{$bc_types[0]}{$i} = $diff ;
                        $candi_pos_bc[$i] = [$pos, $bc_types[0]] ;
                        $hstat{MultPos}++ ;
                    }
                }
            }
            else{
                @bc_types = sort {$ahseqbc->{$b}<=>$ahseqbc->{$a}} @bc_types ;
                my $num = 0 ;
                my $str = "" ;
                my @final_bc = ();
                for my $bc (@bc_types){
                    next if (defined $ahconf_bc->{$bc}) ;
                    $str .= "\t$bc" ;
                    $num++ ;
                    push @final_bc, $bc ;
                }
                if ($num == 0){
                    print OUT "\tMultPos\t", join("\t", @bc_types), "\n" ;
                    $hstat{"M-Null"}++ ;
                }
                elsif ($num == 1){
                    if ($ahbc_stat->{$final_bc[0]} == 1){
                        print OUT "\tM-Uniq$str\n" ;
                        $final_pos_bc[$i] = [$pos, $final_bc[0]] ;
                        $hstat{"M-Uniq"}++ ;
                    }
                    else{
                        print OUT "\tMultPos\t$bc_types[0]\n" ;
                        $hcandi{$bc_types[0]}{$i} = $diff ;
                        $candi_pos_bc[$i] = [$pos, $bc_types[0]] ;
                        $hstat{MultPos}++ ;
                    }
                }
                else {
                    print OUT "\tMultBC$str\n" ;
                    #$final_pos_bc[$i] = [$pos, @final_bc] ;
                    $candi_pos_bc[$i] = [$pos, @final_bc] ;
                    $hstat{MultBC}++ ;
                }
            }
        }
    }
    close(OUT);
    for my $bc (sort {$ahseqbc->{$b}<=>$ahseqbc->{$a}} keys %hcandi){
        my @indexes = sort {$hcandi{$bc}{$a} <=> $hcandi{$bc}{$b}} keys %{$hcandi{$bc}} ;
        if (scalar @indexes == 1){
            my ($pos, @final_bc) = @{$candi_pos_bc[$indexes[0]]} ;
            $final_pos_bc[$indexes[0]] = [$pos, @final_bc] ;
            $ahconf_bc->{$final_bc[0]} = 1 ;
            delete $candi_pos_bc[$indexes[0]] ;
        }
        elsif ($hcandi{$bc}{$indexes[0]} < $hcandi{$bc}{$indexes[1]}){
            my ($pos, @final_bc) = @{$candi_pos_bc[$indexes[0]]} ;
            $final_pos_bc[$indexes[0]] = [$pos, @final_bc] ;
            $ahconf_bc->{$final_bc[0]} = 1 ;
            delete $candi_pos_bc[$indexes[0]] ;
        }
    }
    for (my $i=0; $i<@candi_pos_bc; $i++){
        next if (!defined $candi_pos_bc[$i]) ;
        my ($pos, @bc_types) = @{$candi_pos_bc[$i]} ;
        my @final_bc = ();
        my $flag = 0 ;
        for my $bc (@bc_types){
            next if (defined $ahconf_bc->{$bc}) ;
            push @final_bc, $bc ;
            #if ($ahbc_stat->{$bc} > 1){
            #    $flag = 1 ;
            #}
        }
        $candi_pos_bc[$i] = [$pos, @final_bc] ;
        if (scalar @final_bc == 1){
            $final_pos_bc[$i] = [$pos, @final_bc] ;
            $ahconf_bc->{$final_bc[0]} = 1 ;
            delete $candi_pos_bc[$i] ;
        }
    }
    @{$afinal_flu_pos} = @final_pos_bc ;

    return ;
}

#&out_put(\@final_flu_pos, $outfile);
sub out_put()
{
    my ($afinal_flu_pos, $outfile) = @_ ;

    open (OUT, ">$outfile.barcode.tsv") || die "$outfile.barcode.tsv, $!\n" ;
    open (POS, ">$outfile.barcode_pos.tsv") || die "$outfile.barcode_pos.tsv, $!\n" ;
    for (my $i=0; $i<@{$afinal_flu_pos}; $i++){
        next if (!defined $afinal_flu_pos->[$i]) ;
        my ($pos, @final_bc) = @{$afinal_flu_pos->[$i]} ;
        print OUT $final_bc[0],"\n" ;
        print POS $final_bc[0],"\t$pos\n" ;
    }
    close(OUT);
    close(POS);

    return ;
}

