#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);
use threads;
use threads::shared;

#####################

my %opts;
GetOptions(\%opts,"i=s","k=s","b=s","d=s","p=s","t=s","l=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || !defined($opts{k}) || !defined($opts{b}) || !defined($opts{d}) || !defined($opts{p}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           reads file                              <infile>     must be given
		-k           kmer file                               <infile>     must be given
		-b           bc file                                 <infile>     must be given
		-d           result dir                              <outdir>     must be given
		-p           outfile prefix                          <string>     must be given
		-t           thread num                              [int]        optional [8]
		-l           umi length                              [int]        optional [12]  
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $readsfile = $opts{i} ;
my $kmerfile = $opts{k} ;
my $bcfile = $opts{b} ;
my $outdir = $opts{d} ;
my $prefix = $opts{p} ;
my $threads = defined $opts{t} ? $opts{t} : 8;
my $umi_len = defined $opts{l} ? $opts{l} : 12 ;
mkdir $outdir if (!-d $outdir) ;
$outdir = &ABSOLUTE_DIR($outdir) ;

##############################################################################多线程处理
# 计算文件总行数
my $zcat = "zcat" ;
if ($readsfile =~ /fq$|fastq$/){
    $zcat = "cat" ;
}
my $total_num = `$zcat $readsfile | awk 'END{print NR}'`;
# 每个子文件的行数
my $size = (int($total_num / ($threads * 4)) + 1)*4;
##########快速拆分fastq 数据
`$zcat $readsfile | awk -v sz=$size  'BEGIN{i=1}{ print \$0 > "$outdir/$prefix." i ".tmp.fq"; if (NR>=i*sz){close("$prefix." i ".tmp.fq");i++}}'`;
#######################并行运行
my @files = ();
foreach (1..$threads){
    push @files, "$outdir/$prefix.$_.tmp.fq" ;
}
# 开始多线程
foreach (0..$threads-1){
    my $thr = threads -> create(\&get_bc, $files[$_], $outdir, $prefix, $kmerfile, $bcfile, $size, $_);
}

# join各个线程的结果
my @num_array2;
while(threads -> list()){
    foreach my $t(threads -> list(threads::joinable)){
        my $tmp = $t -> join();
        push @num_array2,$tmp;
    }
}
print "threads:@num_array2\n";
unlink @files;

###########################
######get_bc结果整理合并
`rm $outdir/$prefix.umi` if -e "$outdir/$prefix.umi";
`rm $outdir/$prefix.filter ` if -e "$outdir/$prefix.filter";
`rm $outdir/$prefix.id_map` if -e "$outdir/$prefix.id_map";
`rm $outdir/$prefix.select_id` if -e "$outdir/$prefix.select_id";
my $tmpdir = "$outdir/$prefix.tmpdir" ;
`rm -r $tmpdir` if -d $tmpdir ;
mkdir $tmpdir if (!-d $tmpdir) ;
foreach (0..$threads-1){
	`cat $outdir/$prefix.$_.umi >>$outdir/$prefix.umi`;
	`cat $outdir/$prefix.$_.filter >>$outdir/$prefix.filter`;
	`cat $outdir/$prefix.$_.id_map >>$outdir/$prefix.id_map`;
	`cat $outdir/$prefix.$_.select_id >>$outdir/$prefix.select_id`;
	`rm $outdir/$prefix.$_.umi $outdir/$prefix.$_.filter $outdir/$prefix.$_.id_map $outdir/$prefix.$_.select_id`;
    my @files = glob("$outdir/$prefix.$_.mid_dir/*.umi") ;
    for my $file (@files){
        my $basename = basename($file) ;
        `cat $file >> $tmpdir/$basename` ;
    }
    `rm -r $outdir/$prefix.$_.mid_dir/` ;
}

##########质量统计合并
my @qual_stat = glob "$outdir/$prefix.*.qual.stat";
my ($total_Reads,$valid_barcode,$valid_umi,$final_valid_Reads);
foreach my $qual_stat(@qual_stat){
	chomp $qual_stat;
	open IN,$qual_stat or die $!;
	while(<IN>){
		chomp;
		next if /^\s*$/;
		my ($id,$num,$ratio) = split /\t+/;
		$total_Reads += $num if $id eq "Number of Reads";
		$valid_barcode += $num if $id eq "Valid Barcodes";
		$valid_umi += $num if $id eq "Valid UMIs";
		$final_valid_Reads += $num if $id eq "Final Valid Reads";
	}
	close IN;
}
open OUT, ">$outdir/$prefix.qual.stat" or die $!;
my $valid_barcode_ratio = sprintf "%.2f",$valid_barcode / $total_Reads * 100;
my $valid_umi_ratio = sprintf "%.2f",$valid_umi / $total_Reads * 100;
my $final_valid_Reads_ratio = sprintf "%.2f",$final_valid_Reads / $total_Reads * 100;
print OUT "Number of Reads\t$total_Reads\t100\n";
print OUT "Valid Barcodes\t$valid_barcode\t$valid_barcode_ratio\n";
print OUT "Valid UMIs\t$valid_umi\t$valid_umi_ratio\n";
print OUT "Final Valid Reads\t$final_valid_Reads\t$final_valid_Reads_ratio\n";
close OUT;
##########barcode统计合并
my @bc_stat = glob "$outdir/$prefix.*.bc_stat";
my %hbc_stat = ();
foreach my $bc_stat(@bc_stat){
	chomp $bc_stat;
	open IN,$bc_stat or die $!;
	while(<IN>){
		chomp;
		next if /^\s*$|^\#/;
		last if /^\===/;
		my ($id,$num,$ratio) = split;
        $hbc_stat{$id} += $num ;
	}
	close IN;
}
open OUT, ">$outdir/$prefix.bc_stat" or die $!;
print OUT "#Type\tReadsNum\tPercent(%)\n";
my %hsingle = ();
for my $type (sort keys %hbc_stat){
    my $num = $hbc_stat{$type} ;
    my $ratio = sprintf "%.2f", $num / $total_Reads * 100;
    print OUT "$type\t$num\t$ratio\n" ;
    while($type =~ m/(bc\d)/g){
        $hsingle{$1} += $num ;
    }
}
print OUT "======== single stat ========\n";
for my $type (sort keys %hsingle){
    my $num = $hsingle{$type} ;
    my $ratio = sprintf "%.2f", $num / $total_Reads * 100;
    print OUT "$type\t$num\t$ratio\n" ;
}
close OUT;
##########stat统计合并
my @stat;
for(my $i = 0;$i < $threads;$i++){
	push @stat,"$outdir/$prefix.$i.stat";
}
my ($full,$part,$null,$umi,$final);
foreach my $stat(@stat){
	chomp $stat;
	open IN,$stat or die $!;
	while(<IN>){
		chomp;
		next if /^\s*$/;
		next if /^===/;
		my ($id,$num,$ratio) = split;
		$full += $num if $id eq "Full";
		$part += $num if $id eq "Part";
		$umi += $num if $id eq "UMI";
		$final += $num if $id eq "Final";
	}
	close IN;
}
my $full_ratio = sprintf "%.2f",$full / $total_Reads * 100;
my $part_ratio = sprintf "%.2f",$part / $total_Reads * 100;
my $umi_ratio = sprintf "%.2f",$umi / $total_Reads * 100;
my $final_ratio = sprintf "%.2f",$final / $total_Reads * 100;
$null = $total_Reads - $full - $part;
my $null_ratio = sprintf "%.2f",$null / $total_Reads * 100;
open OUT, ">$outdir/$prefix.stat" or die $!;
print OUT "Total\t$total_Reads\t100\n";
print OUT "Full\t$full\t$full_ratio\n";
print OUT "Part\t$part\t$part_ratio\n";
print OUT "Null\t$null\t$null_ratio\n";
print OUT "============\n";
print OUT "UMI\t$umi\t$umi_ratio\n";
print OUT "Final\t$final\t$final_ratio\n";
close OUT;

#########清理中间文件
for(my $i = 0;$i < $threads;$i++){
	`rm $outdir/$prefix.$i.qual.stat`;
	`rm $outdir/$prefix.$i.bc_stat`;
	`rm $outdir/$prefix.$i.stat`;
}

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

#my $thr = threads -> create(\&get_bc, $files[$_], $outdir, $prefix, $kmerfile, $bcfile, $size, $_);
sub get_bc()
{
    my ($readsfile, $outdir, $prefix, $kmerfile, $bcfile, $file_line, $num) = @_ ;

    my $start_index = $file_line/4*$num ;
    my $cmd = "perl $Bin/get_reads_bcV2_type_v2.4.pl -i $readsfile -k $kmerfile -b $bcfile -d $outdir -p $prefix.$num -x $start_index" ;
    &run_or_die($cmd) ;

    return ($num);
}

