#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","o=s","l=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           umi file                           <infile>     must be given
		-o           out prefix                         <outfile>    must be given
		-l           umi length                         [int]        optional[12]  
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $umifile = $opts{i} ;
my $outfile = $opts{o} ;
my $umi_len = defined $opts{l} ? $opts{l} : 12 ;

## get umi
my %hbc_umi = ();
&read_and_get_umi($umifile, \%hbc_umi);

## correct and out put
my %hbc_stat = ();
&correct_umi(\%hbc_umi, $umi_len, $outfile, \%hbc_stat);

## out put bc stat
&output_bc_stat(\%hbc_stat, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&read_and_get_umi($umifile, \%hbc_umi);
sub read_and_get_umi()
{
    my ($umifile, $ahbc_umi) = @_ ;

    open (IN, $umifile) || die "$umifile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$|^\#/);
        my ($id, $bc, $cigar, $pos, $umi_seq, $umi_pos) = split ;
        push @{$ahbc_umi->{$bc}{$umi_seq}}, $id ;
    }
    close(IN);

    return ;
}

#&correct_umi(\%hbc_umi, $umi_len, $outfile, \%hbc_stat);
sub correct_umi()
{
    my ($ahbc_umi, $umi_len, $outfile, $ahbc_stat) = @_ ;

    open (OUT, ">$outfile.bc_umi_read.tsv") || die "$outfile.bc_umi_read.tsv, $!\n" ;
    open (ID, ">$outfile.bc_umi_read.tsv.id") || die "$outfile.bc_umi_read.tsv.id, $!\n" ;
    open (COR, ">$outfile.umi_cor.info") || die "$outfile.umi_cor.info, $!\n" ;
    my $pre_len = int($umi_len/2) ;
    my $aft_len = $umi_len - $pre_len ;
    for my $bc (keys %{$ahbc_umi}){
        my @umis = sort {scalar(@{$ahbc_umi->{$bc}{$b}})<=>scalar(@{$ahbc_umi->{$bc}{$a}}) || $a cmp $b} keys %{$ahbc_umi->{$bc}} ;
        my %hpre_umi = ();
        my %haft_umi = ();
        my %humi = ();
        for my $umi (@umis){
            my $pre_seq = substr($umi, 0, $pre_len) ;
            my $aft_seq = substr($umi, $pre_len, $aft_len) ;
            if (!defined $hpre_umi{$pre_seq} && !defined $haft_umi{$aft_seq}){
                push @{$hpre_umi{$pre_seq}}, $aft_seq ;
                push @{$haft_umi{$aft_seq}}, $pre_seq ;
                $humi{$umi} = [@{$ahbc_umi->{$bc}{$umi}}] ;
            }
            elsif(defined $hpre_umi{$pre_seq}){
                my $flag = 0 ;
                for my $seq (@{$hpre_umi{$pre_seq}}){
                    my $mask = $aft_seq ^ $seq ;
                    my $num = () = $mask =~ m/[^\0]/g ;
                    if ($num == 1){
                        my $pre_umi = $pre_seq.$seq ;
                        push @{$humi{$pre_umi}}, @{$ahbc_umi->{$bc}{$umi}} ;
                        print COR "$bc\t$umi -> $pre_umi\n" ;
                        $flag = 1 ;
                        last ;
                    }
                }
                if ($flag == 0){
                    $humi{$umi} = [@{$ahbc_umi->{$bc}{$umi}}] ;
                }
            }
            else{
                my $flag = 0 ;
                for my $seq (@{$haft_umi{$aft_seq}}){
                    my $mask = $pre_seq ^ $seq ;
                    my $num = () = $mask =~ m/[^\0]/g ;
                    if ($num == 1){
                        my $pre_umi = $seq.$aft_seq ;
                        push @{$humi{$pre_umi}}, @{$ahbc_umi->{$bc}{$umi}} ;
                        print COR "$bc\t$umi -> $pre_umi\n" ;
                        $flag = 1 ;
                        last ;
                    }
                }
                if ($flag == 0){
                    $humi{$umi} = [@{$ahbc_umi->{$bc}{$umi}}] ;
                }
            }
        }
        for my $umi (keys %humi){
            my $num = scalar @{$humi{$umi}} ;
            print OUT "$bc\t$umi\t$num\n" ;
            print ID "$bc\t$umi" ;
            for my $id (@{$humi{$umi}}){
                print ID "\t$id" ;
            }
            print ID "\n" ;
            $ahbc_stat->{$bc}{umi} ++ ;
            $ahbc_stat->{$bc}{read} += $num ;
        }
        delete $ahbc_umi->{$bc} ;
    }
    close(OUT) ;
    close(ID);
    close(COR);

    return ;
}

#&output_bc_stat(\%hbc_stat, $outfile);
sub output_bc_stat()
{
    my ($ahbc_stat, $outfile) = @_ ;

    open (OUT, ">$outfile.full_stat") || die "$outfile.full_stat, $!\n" ;
    print OUT "#Type\tReadsNum\tUMI\n" ;
    my %hstat = ();
    for my $type (sort {$ahbc_stat->{$b}{read} <=> $ahbc_stat->{$a}{read} || $a cmp $b} keys %{$ahbc_stat}){
        my $read_num = $ahbc_stat->{$type}{read} ;
        my $umi_num = $ahbc_stat->{$type}{umi} ;
        print OUT "$type\t$read_num\t$umi_num\n" ;
        my @bcs = split /\-/, $type ;
        for my $bc (@bcs){
            if ($bc =~ m/(bc\d)\_\d+/){
                $hstat{$1}{$bc}{read} += $read_num ;
                $hstat{$1}{$bc}{umi} += $umi_num ;
            }
        }
    }
    close(OUT);

    open (OUT, ">$outfile.bc_dist") || die "$outfile.bc_dist, $!\n" ;
    print OUT "#Type\tBarcode\tNumber\tUMI\n" ;
    for my $type (sort keys %hstat){
        for my $bc (sort {$hstat{$type}{$b}{read}<=>$hstat{$type}{$a}{read}} keys %{$hstat{$type}}){
            my $read_num = $hstat{$type}{$bc}{read} ;
            my $umi_num = $hstat{$type}{$bc}{umi} ;
            print OUT "$type\t$bc\t$read_num\t$umi_num\n" ;
        }
    }
    close(OUT);

    return ;
}

