#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);
use threads;
use threads::shared;

#####################

my %opts;
GetOptions(\%opts,"i=s","d=s","o=s","l=s","t=s","h" );

#&help()if(defined $opts{h});
if((!defined($opts{i}) && !defined($opts{d})) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           umi file                           <infile>     optional with -d
		-d           umi file dir                       <indir>      optional with -i
		-o           out prefix                         <outfile>    must be given
		-l           umi length                         [int]        optional[12]  
		-t           threads num                        [int]        optional[8]  
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $outfile = $opts{o} ;
my $umi_len = defined $opts{l} ? $opts{l} : 12 ;
my $threads_num = defined $opts{t} ? $opts{t} : 8 ;
my $indir ;
my @files = ();
if (defined $opts{d}){
    $indir = $opts{d} ;
    @files = glob("$indir/*.umi");
}
elsif (defined $opts{i}){
    my $umifile = $opts{i} ;
    ($indir, @files) = &split_umi($umifile, $outfile);
}

## correct and out put
my $index = 0 ;
my $file_num = scalar(@files);
while(1){
    last if ($index >= $file_num);
    while(scalar(threads->list()) < $threads_num && $index < $file_num){
        my $file = $files[$index] ;
        threads->new(\&correct_umi, $file, $umi_len);
        $index++ ;
    }
    for my $thread (threads->list(threads::all)){
        if ($thread->is_joinable()){
            $thread->join();
        }
    }
}
for my $thread (threads->list(threads::all)){
    $thread->join();
}

## out put bc stat
&merge_result($indir, \@files, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&split_umi($umifile, $outfile);
sub split_umi()
{
    my ($umifile, $outfile) = @_ ;

    my $outdir = "$outfile\_tmpdir" ;

    mkdir $outdir if (!-d $outdir) ;
    my @files = ();

    open (IN, $umifile) || die "$umifile, $!\n" ;
    my %hfd = ();
    my %hbc2fd = ();
    my $bc1_num = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$|^\#/);
        my ($id, $bc, $cigar, $pos, $umi_seq, $umi_pos) = split ;
        my ($bc1, $bc2, $bc3) = split /\-/, $bc ;
        if (!defined $hbc2fd{$bc1}){
            my $index = int($bc1_num/12) ;
            $hbc2fd{$bc1} = $index ;
            if (!defined $hfd{$index}){
                open ($hfd{$index}, ">$outdir/$index.umi") || die "$outdir/$index.umi, $!\n" ;
                push @files, "$outdir/$index.umi" ;
            }
            $bc1_num++ ;
        }
        print {$hfd{$hbc2fd{$bc1}}} "$_\n" ;
    }
    close(IN);
    for my $index (keys %hfd){
        close($hfd{$index}) ;
    }
    @files = sort @files ;

    return($outdir, @files);
}

#threads->new(\&correct_umi, $file, $umi_len);
sub correct_umi()
{
    my ($file, $umi_len) = @_ ;

    (my $midfile = $file) =~ s/.umi$// ;
    my $cmd = "perl $Bin/umi_correct.pl -i $file -o $midfile -l $umi_len" ;
    &run_or_die($cmd) ;

    return ;
}
 
#&merge_result($tmpdir, \@files, $outfile);
sub merge_result()
{
    my ($tmpdir, $afiles, $outfile) = @_ ;

    `rm $outfile.bc_umi_read.tsv` if (-e "$outfile.bc_umi_read.tsv") ;
    `rm $outfile.bc_umi_read.tsv.id` if (-e "$outfile.bc_umi_read.tsv.id") ;
    `rm $outfile.umi_cor.info` if (-e "$outfile.umi_cor.info") ;
    `rm $outfile.full_stat` if (-e "$outfile.full_stat") ;
    `rm $outfile.bc_dist` if (-e "$outfile.bc_dist") ;
    my %hfull_stat = ();
    my %hbc_dist = ();
    for my $file (@{$afiles}){
        $file =~ s/.umi$// ;
        `cat $file.bc_umi_read.tsv >> $outfile.bc_umi_read.tsv` ;
        `cat $file.bc_umi_read.tsv.id >> $outfile.bc_umi_read.tsv.id` ;
        `cat $file.umi_cor.info >> $outfile.umi_cor.info` ;
        &get_full_stat_info("$file.full_stat", \%hfull_stat);
        &get_bc_dist_info("$file.bc_dist", \%hbc_dist);
        `rm $file.bc_umi_read.tsv* $file.umi_cor.info $file.full_stat $file.bc_dist` ;
    }
    open (OUT, ">$outfile.full_stat") || die "$outfile.full_stat, $!\n" ;
    print OUT "#Type\tReadsNum\tUMI\n" ;
    for my $type (sort {$hfull_stat{$b}[0] <=> $hfull_stat{$a}[0] || $a cmp $b} keys %hfull_stat){
        my ($read_num, $umi_num) = @{$hfull_stat{$type}} ;
        print OUT "$type\t$read_num\t$umi_num\n" ;
    }
    open (OUT, ">$outfile.bc_dist") || die "$outfile.bc_dist, $!\n" ;
    print OUT "#Type\tBarcode\tNumber\tUMI\n" ;
    for my $type (sort keys %hbc_dist){
        for my $bc (sort {$hbc_dist{$type}{$b}[0]<=>$hbc_dist{$type}{$a}[0]} keys %{$hbc_dist{$type}}){
            my ($read_num, $umi_num) = @{$hbc_dist{$type}{$bc}} ;
            print OUT "$type\t$bc\t$read_num\t$umi_num\n" ;
        }
    }
    close(OUT);

    if (defined $opts{i}){
        `rm -r $tmpdir` ;
    }

    return ;
}

#&get_full_stat_info("$file.full_stat", \%hfull_stat);
sub get_full_stat_info()
{
    my ($infile, $ahfull_stat) = @_ ;

    open (IN, $infile) || die "$infile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#/);
        my ($bc, $rd, $umi) = split ;
        $ahfull_stat->{$bc} = [$rd, $umi] ;
    }
    close(IN);

    return ;
}

#&get_bc_dist_info("$file.bc_dist", \%hbc_dist);
sub get_bc_dist_info()
{
    my ($infile, $ahbc_dist) = @_ ;

    open (IN, $infile) || die "$infile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#/);
        my ($type, $bc, $rd, $umi) = split ;
        $ahbc_dist->{$type}{$bc}[0] += $rd ;
        $ahbc_dist->{$type}{$bc}[1] += $umi ;
    }
    close(IN);

    return ;
}

