suppressMessages(library(getopt))
spec=matrix(c(
  'input','i',1,"character",'directory,containing features.csv,level_x_barcode_pos.csv,level_x_matrix.csv',
  'outdir','o',1,"character",'Output directory',
  'number','n',1,'numeric','1 for gene id,2 for gene name',
  'MinGene','g',1,"numeric","Include cells where at least this many features are detected,e.g 500",
  'MinCell','c',1,"numeric","Include features detected in at least this many cells,e.g 5",
  'resolution','r',1,"numeric","Value of the resolution parameter,e.g 0.5",
  'help','h',0,'logical','print usage'
),byrow = T,ncol = 5)
opt=getopt(spec)
USAGE=function(){
  cat("Program: ReduceAndCluster.R
       Version: v1.0
       Contact: litt <litt@biomarker.com.cn>
       Date:  20220221
       Description: This program is for reducing dimension and clustering\n")
  cat(paste(getopt(spec, usage = TRUE), sep = "\n"))
  q()
}
if(is.null(opt$outdir) | is.null(opt$input) | is.null(opt$MinGene) | is.null(opt$MinCell) | is.null(opt$number) |is.null(opt$resolution) | !is.null(opt$help)){USAGE()}
suppressMessages({
  library(Seurat)
  library(dplyr)
  library(tibble)
  library(ggplot2)
  
})
StartTime=proc.time()
##函数
#输出ggplot对象
SavePlot <- function(od, filename, data, width = 8, height = 8){
  file.png <- paste(filename, "png", sep = ".")
  file.pdf <- paste(filename, "pdf", sep = ".")
  ggsave(filename = file.path(od, file.png), plot = data, width = width, height = height)
  ggsave(filename = file.path(od, file.pdf), plot = data, width = width, height = height)
}
# #输入参数:表达矩阵，
#resolution
res=as.numeric(opt$resolution)
print(paste0("res:",class(res)))
print(res)
FilePath=opt$input
#过滤参数：细胞数5，基因数500
MinCell=as.numeric(opt$MinCell)#5
MinFeatures=as.numeric(opt$MinGene)#500
print("MinFeatures")
print(MinFeatures)
#输出目录
outdir=opt$outdir#
if(!dir.exists(outdir)){dir.create(path=outdir)}
#读取表达矩阵
print("hello")
expr=Read10X(opt$input,gene.column = as.numeric(opt$number))
print("world")
object=CreateSeuratObject(counts = expr,assay = "Spatial",min.cells=MinCell,min.features=MinFeatures)
#object <- SCTransform(object, assay = "Spatial", verbose = FALSE,return.only.var.genes = FALSE)

if(FALSE){
object1=object %>%
  RunPCA(verbose = F,assay = "SCT") %>%
  FindNeighbors(reduction = "pca",dims = 1:30,verbose = F) %>% #因为SC标准化故dims选30
  FindClusters(verbose = F,resolution = res) %>%#resolution可调，影响cluster数目
  RunUMAP(reduction = "pca",dims = 1:30) %>%
  RunTSNE(reduction = "pca",dims = 1:30)
}
object1 <- NormalizeData(object, normalization.method = "LogNormalize", scale.factor = 10000)
object1 <- FindVariableFeatures(object1, selection.method = "vst", nfeatures = 2000)
object1 <- ScaleData(object1)
object1 <- RunPCA(object=object1,pc.genes = VariableFeatures(object1))
object1 <- FindNeighbors(object=object1, reduction = "pca",dims = 1:30,verbose = F)
object1 <- FindClusters(object=object1, verbose = F,resolution = res)
object1 <- RunUMAP(object=object1, reduction = "pca",dims = 1:30)
#object1 <- RunTSNE(object=object1, reduction = "pca",dims = 1:30)


Cluster=object1@meta.data %>%
  select(.,seurat_clusters) %>%
  rownames_to_column(.,var = "Barcode") %>%
  arrange(.,seurat_clusters)
write.csv(Cluster,file.path(outdir,"cluster.csv"),quote = F,row.names = F)
cluster_freq <- as.data.frame(table(Cluster$seurat_clusters))
colnames(cluster_freq) <- c("cluster", "cells_num")
write.csv(cluster_freq,file.path(outdir,"cluster_cells_num.csv"),quote=F, row.names=F)
#
umapPlot=UMAPPlot(object1, label = F)
SavePlot(outdir,"UMAP",umapPlot)

object1.markers <- FindAllMarkers(object1, only.pos = FALSE, min.pct = 0.25)
#top50 <- object1.markers %>% group_by(cluster) %>% top_n(n = 50, wt = avg_log2FC)
marker <- object1.markers %>% group_by(cluster)
write.csv(marker,file.path(outdir,"marker_gene.csv"),quote = F, row.names = F)
saveRDS(object1, file = file.path(outdir,"object.RDS"))

# 晓蕾改动2022.9.23
# object1.markers <- FindAllMarkers(object1, only.pos = TRUE, min.pct = 0.25)
# top10 <- object1.markers %>% group_by(cluster) %>% top_n(n = 10, wt = avg_log2FC)
# write.csv(top10,file.path(outdir,"marker_gene.csv"),quote = F, row.names = F)
# 晓蕾改动2022.9.23
#tsnePlot=TSNEPlot(object1, label = F)
#SavePlot(outdir,"t-SNE",tsnePlot)
EndTime=proc.time()
RunTime <- EndTime - StartTime
print(paste0('ProcessingTime: ',RunTime[3][[1]],'seconds'))

# print("is ok ###########################")
# cluster1.markers <- FindMarkers(object1, ident.1 = 1, min.pct = 0.25)
# head(cluster1.markers, n = 10)
# # 找出每个cluster的标记与所有剩余的细胞相比较，只报告阳性细胞
# pbmc.markers <- FindAllMarkers(object1, only.pos = TRUE, min.pct = 0.25, logfc.threshold = 0.25)
# pbmc.markers %>% group_by(cluster) %>% top_n(n = 5, wt = avg_log2FC)
# print(pbmc.markers)
#
# #找出区分cluster 5与cluster 0和cluster 3的所有标记
# cluster5.markers <- FindMarkers(pbmc, ident.1 = 5, ident.2 = c(0, 3), min.pct = 0.25)
# head(cluster5.markers, n = 5)
