import os
os.environ["MKL_NUM_THREADS"] = "4"
os.environ["NUMEXPR_NUM_THREADS"] = "4"
os.environ["OMP_NUM_THREADS"] = "4"
import cv2
import random
import copy
import time
import numpy as np
import tifffile
# import matplotlib.pyplot as plt
import cellpose.io
import cellpose.utils
import cellpose.models
import cellpose.plot
import argparse
import yaml
from scipy import ndimage as ndi
from skimage.segmentation import watershed
import pandas as pd 
import json
from  HERegionSearch import get_img_conts, get_img_conts_manual

import sys
base_dir = os.path.split(os.path.realpath(sys.argv[0]))[0]

import logging
if "--save_dir" in sys.argv or '--barcode_save_dir' in sys.argv:
    pass
    if "--save_dir" in sys.argv:
        log_dir = sys.argv[sys.argv.index('--save_dir') + 1]
    else:
        log_dir = sys.argv[sys.argv.index('--barcode_save_dir') + 1]
    if not os.path.exists(log_dir):
        os.mkdir(log_dir)
    logging.basicConfig(filename=os.path.join(log_dir, "SegtoBarcode.log"), level=logging.DEBUG,
                        format='%(asctime)s %(message)s')
else:
    logging.basicConfig(filename=__file__.replace(".py", ".log"), level=logging.DEBUG, format='%(asctime)s %(message)s')
LOG = logging


def write(message):
    if message == "\n":
        std_write(message)  # 控制台标准输出
        return
    if "segModel" in globals() and segModel.save_dir is not None:
        message = os.path.split(segModel.save_dir)[0] + "---" + message
    std_write(message)  # 控制台标准输出
    LOG.info(message)


# 辅助功能 日志，显示
std_write = copy.deepcopy(sys.stdout.write)
sys.stdout.write = write


class CellSeg:

    def __init__(self,diameter,seg_channel,inversion,
                 flow_threshold,min_detect_value,max_detect_value,
                 overlap,sub_size,img_path,model_path,save_dir,conts_filter=None,
                 nucleus_base_path=None, enhance=False):
        '''读取参数'''
        self.diameter = diameter
        self.seg_channel = seg_channel
        self.conts_filter = conts_filter
        self.channels = [[seg_channel+1, 0]]
        self.inversion = inversion
        self.flow_threshold = flow_threshold
        self.min_detect_value = min_detect_value
        self.max_detect_value = max_detect_value
        self.overlap = overlap
        self.sub_size = sub_size
        self.img_path = img_path
        self.nucleus_base_path = nucleus_base_path
        self.nucleus_base = None
        self.enhance = enhance
        self.model_path = model_path
        if self.model_path.startswith("./"):
            self.model_path = os.path.join(base_dir, self.model_path.lstrip('./'))
        print(f"model path:{self.model_path}")
        if not save_dir:
            self.save_dir = os.path.splitext(img_path)[0]
        else:
            self.save_dir = save_dir
        if not os.path.exists(self.save_dir):
            os.mkdir(self.save_dir)


    def save_progress(self,txt_path,progress):
        '''
        读取分割保存进度
        '''
        
        progress_path = os.path.join(txt_path,'progress.txt')
        print(progress_path)
        progerss_file = open(progress_path,'w')
        print(f'progressing:{progress}%......')
        progerss_file.write(str(progress))
        progerss_file.close()

    def roi_mask(roi_group):
        '''
        增添ROI区域遮罩
        '''

        return 0

    def create_model(self):
        '''返回创建模型'''
        # model = cellpose.models.CellposeModel(pretrained_model=self.model_path, gpu=True, diam_mean=self.diameter)
        model = cellpose.models.CellposeModel(pretrained_model=self.model_path, gpu=True)
        if not os.path.exists(self.save_dir):
            os.mkdir(self.save_dir)
        self.save_progress(self.save_dir,0)
        return model
    
    def load_image(self):
        '''读取图片获得保存荧光图 返回原尺寸mask'''
        img = cellpose.io.imread(self.img_path)
        if len(img.shape) == 2:
            img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
        # img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        tifffile.imwrite(os.path.join(self.save_dir, "fluorescence.tif"),
                         cv2.cvtColor(img[..., self.seg_channel], cv2.COLOR_GRAY2BGR), compression="jpeg")
        img = cv2.cvtColor(img[..., self.seg_channel], cv2.COLOR_GRAY2BGR)
        cell_mask = np.zeros(img.shape[:2], dtype=bool)
        # 预处理数据加载
        if self.nucleus_base_path is not None:
            self.nucleus_base = np.load(self.nucleus_base_path)  # 预分割结果，后续分割会在此基础上继续操作
            if self.nucleus_base.shape[:2] != img.shape[:2]:
                raise Exception(f"npy shape {self.nucleus_base.shape[:2]} is not match img shape{img.shape[:2]}")
            cell_mask[self.nucleus_base > 0] = True  # 标记已有的细胞区域为true，后续将不在分割
        else:
            self.nucleus_base = None
        if self.conts_filter != 'None':  # 不加轮廓
            if self.conts_filter in ("fl", "he"):  # 自动识别he或fl边界
                filter_conts = []
                if args.fl_gray_num == 0:
                    filter_conts = get_img_conts(img, self.conts_filter)  # 找荧光图组织边界
                else:
                    filter_conts = get_img_conts_manual(img, args.fl_gray_num, self.conts_filter)  # 找荧光图组织边界
                conts_binary = np.zeros(img.shape[:2], dtype=np.uint8)
                cv2.drawContours(conts_binary, filter_conts, -1, 255, -1)
            elif self.conts_filter.endswith(".json"):   # 传入给定边界
                with open(self.conts_filter, 'r') as fp:
                    roi_json = json.load(fp)
                roi = roi_json["ori_group"]
                filter_conts = [np.array(sub_roi) for sub_roi in roi]
                conts_binary = np.zeros(img.shape[:2], np.uint8)
                conts_binary = cv2.drawContours(conts_binary, filter_conts, -1, 255, -1)
                if "negative_roi_group" in roi_json:
                    roi_remove = roi_json["negative_roi_group"]
                    conts_binary = cv2.drawContours(conts_binary, np.asarray(roi_remove), -1, 0, -1)
            else:
                raise Exception("Wrong conts_filter input!")

            cell_mask[conts_binary == 0] = True
            del conts_binary

            # 画一下边界小图，以供后续排查
            img_with_conts = img.copy()
            cv2.drawContours(img_with_conts, filter_conts, -1, (0, 255, 0), 10)
            tifffile.imwrite(os.path.join(self.save_dir, "conts.tif"), cv2.resize(img_with_conts, (5000, 5000)),
                             compression="jpeg")
            del img_with_conts
        return img, cell_mask
     
    def seg_image(self,img,cell_mask,model):
        '''分割子图'''
        print(type(self.inversion),self.inversion)
        if self.inversion==True:
            img = 255 - img
            print('image values have been inversed!')
        count = 0
        total_times = len(range(0, img.shape[0], self.sub_size[0]-self.overlap[0])) * len(range(0, img.shape[1], self.sub_size[1]-self.overlap[1]))

        # 直方图局部均衡化算子
        # cla = cv2.createCLAHE()
        #
        # 增加多尺度计算
        if type(self.diameter) != list:
            self.diameter = [self.diameter]

        for y_start in range(0, img.shape[0], self.sub_size[0]-self.overlap[0]):
            for x_start in range(0, img.shape[1], self.sub_size[1]-self.overlap[1]):
                t1 = time.time()
                slice_y = slice(y_start, y_start + self.sub_size[1])
                slice_x = slice(x_start, x_start + self.sub_size[0])
                tmp_mask = cell_mask[slice_y, slice_x, ...]  # 取二值化mask
                img_sub = img[slice_y, slice_x, ...]
                tmp_img = img_sub.copy()  # 待分割子图,取副本，因为np是指针形式，会改变原图
                tmp_img[tmp_img < self.min_detect_value] = 0  # 做低像素值截断
                tmp_img[tmp_img > self.max_detect_value] = self.max_detect_value  # 做高像素值截断

                mask_sub = np.zeros_like(tmp_mask, dtype=int)
                for sub_index, diameter in enumerate(self.diameter):
                    tmp_img[tmp_mask] = 0  # 将已有的mask区域置0， 不参与cellpose计算
                    if self.enhance:  # 增强模式
                        # 直方图局部均衡化
                        clahe = cv2.createCLAHE(clipLimit=4.0, tileGridSize=(16, 16))
                        if len(tmp_img.shape) == 2:
                            # tmp_img = cv2.equalizeHist(tmp_img)
                            tmp_img = clahe.apply(tmp_img)
                        else:
                            for tmp_channel in range(tmp_img.shape[-1]):
                                # tmp_img[..., tmp_channel] = cv2.equalizeHist(tmp_img[..., tmp_channel])
                                tmp_img[..., tmp_channel] = clahe.apply(tmp_img[..., tmp_channel])
                    #
                    tifffile.imwrite(os.path.join(self.save_dir, "{}_{}_ori_{}.tif".format(y_start, x_start, sub_index)), tmp_img, compression="jpeg")  # 存一份原图，用于对照
                    mask_tmp, flow_sub, style_sub = model.eval([tmp_img], diameter=diameter, \
                                                               channels=self.channels, flow_threshold=self.flow_threshold)
                    mask_tmp = mask_tmp[0]
                    mask_tmp = cellpose.utils.remove_edge_masks(mask_tmp)
                    tmp_mask[mask_tmp > 0] = True

                    mask_tmp[mask_tmp > 0] += mask_sub.max()
                    mask_sub[...] = np.where(mask_sub > 0, mask_sub, mask_tmp)

                print(f"{y_start}_{x_start} time cost: {time.time() - t1}")
                outline = cellpose.utils.masks_to_outlines(mask_sub)

                img_sub[outline, ...] = 255
                tifffile.imwrite(os.path.join(self.save_dir, "{}_{}.tif".format(y_start, x_start)), img_sub, compression="jpeg")
                np.save(os.path.join(self.save_dir, "{}_{}.npy".format(y_start, x_start)), mask_sub)
                count += 1
                self.save_progress(self.save_dir,(90/total_times)*count)
        tifffile.imwrite(os.path.join(self.save_dir, "all_outline.tif"), img[...,self.seg_channel], compression="jpeg")
        return img
   
    def splice_masks(self,img):
        '''拼接分割子图mask'''
        print("Start fused sub masks...")
        if self.nucleus_base is not None:
            mask = self.nucleus_base
        else:
            mask = np.zeros(img.shape[:2], dtype=np.int32)
        for y_start in range(0, img.shape[0], self.sub_size[0]-self.overlap[0]):
            for x_start in range(0, img.shape[1], self.sub_size[1]-self.overlap[1]):
                slice_y = slice(y_start, y_start + self.sub_size[1])
                slice_x = slice(x_start, x_start + self.sub_size[0])
                mask_sub = mask[slice_y, slice_x]

                mask_tmp_path = os.path.join(self.save_dir, "{}_{}.npy".format(y_start, x_start))
                mask_tmp = np.load(mask_tmp_path).astype(np.int32)

                print("Fused mask: {}, sub_cell:{}, total cell:{}".format(os.path.join(self.save_dir,
                                                                                    "{}_{}.npy".format(y_start, x_start)),
                                                                        mask_tmp.max(), mask.max()))

                mask_tmp[mask_tmp > 0] += mask.max()
                mask_sub[...] = np.where(mask_sub > 0, mask_sub, mask_tmp)

        self.save_progress(self.save_dir,100)
        nucleus_path = os.path.join(self.save_dir, "nucleus.npy")
        np.save(nucleus_path, mask)
        return nucleus_path


class DrawCell:
   
    def __init__(self,cell_edge, calculate_rate, nucleus_path, save_dir=None, cells_npy="cells.npy"):
        self.cell_edge = cell_edge # from yaml import 
        self.calculate_rate = calculate_rate
        self.nucleus_path = nucleus_path
        if save_dir is None:
            self.save_dir = os.path.split(nucleus_path)[0]
        else:
            self.save_dir = save_dir
        self.cells_name = cells_npy
        pass
    
    def hsv_to_rgb(self,h, s, v):
        '''hsv转rgb'''
        if s == 0.0:
            return v, v, v
        i = int(h * 6.0)  # XXX assume int() truncates!
        f = (h * 6.0) - i
        p = v * (1.0 - s)
        q = v * (1.0 - s * f)
        t = v * (1.0 - s * (1.0 - f))
        i = i % 6
        if i == 0:
            return v, t, p
        if i == 1:
            return q, v, p
        if i == 2:
            return p, v, t
        if i == 3:
            return p, q, v
        if i == 4:
            return t, p, v
        if i == 5:
            return v, p, q

    def show_img(pic, name=None, line_width=3):
        '''展示图片'''
        def mouse(event, x, y, flags, param):
            if event == cv2.EVENT_LBUTTONDOWN:
                print(x, y, pic[y, x])
            if event == cv2.EVENT_RBUTTONDOWN:
                param[:] = [x, y]
                cv2.waitKey(300)
                cv2.destroyWindow(name)
            if event == cv2.EVENT_MOUSEMOVE:
                n_pic = cv2.line(copy.copy(pic), (x, 0), (x, pic.shape[0]), (0, 255, 0), line_width)
                n_pic = cv2.line(n_pic, (0, y), (pic.shape[1], y), (0, 255, 0), line_width)
                cv2.imshow(name, n_pic)

        if not name:
            name = str(random.random())
        cv2.namedWindow(name, flags=cv2.WINDOW_NORMAL)
        cv2.imshow(name, pic)
        tmp = []
        cv2.setMouseCallback(name, mouse, tmp)
        cv2.waitKey()
        try:
            cv2.destroyWindow(name)
        except:
            pass
        return tmp[::-1]

    def generate_cells(self):
        '''加载nucleus.npy,生成细胞分割边界labels, 储存为cells.npy,以及存储路径'''
        masks = np.load(self.nucleus_path)
        mask_rs = ndi.zoom(masks, 1 /self.calculate_rate, order=0)  # 根据倍率减少计算尺寸
        nucleus = np.where(mask_rs > 0, 255, 0).astype(dtype=np.uint8)  # 先拿到核的二值化图像        #
        distance = ndi.distance_transform_edt(cv2.bitwise_not(nucleus))  # 计算核边界的距离图
        # distance_save_dir = os.path.join(self.save_dir,'distance.npy')
        # np.save(distance_save_dir,distance)
        # # distance_mask = distance
        cells = watershed(distance, mask_rs, mask=(distance <= (self.cell_edge // self.calculate_rate)))  # 分水岭得到细胞分割结果
        a = cellpose.utils.masks_to_outlines(cells)  # 拿到细胞分割边界
        cells_outlines = cells * a
        # plt.imshow(labels_outlines + mask_rs, cmap=plt.cm.flag)
        # plt.show()
        cells = ndi.zoom(cells, np.asarray(masks.shape) / cells.shape, order=0)
        cells_path = os.path.join(self.save_dir, self.cells_name)
        np.save(cells_path, cells)
        return masks, cells, cells_path

    def cells_coloring(self,masks,cells):
        '''生成colors.npy 返回路径'''

        colors_path = os.path.join(self.save_dir, "colors.npy")

        if os.path.exists(colors_path) and not args.redo:
            colors = np.load(colors_path)
            print("Color is exists!, use it.")
        else:
            colors = np.asarray([self.hsv_to_rgb(np.random.rand()*0.5+0.5, np.random.rand()*0.5+0.5, 1.0) for x in range(cells.max() + 1)])
            colors = (colors*255).astype(np.uint8)
            colors[0] = [0, 0, 0]   # 0置为黑色
            
            np.save(colors_path, colors)
        # 画可视化图像
        nucleus_color_img = colors[masks]
        nucleus_color_img_path = os.path.join(self.save_dir, "nucleus_color.tif")
        tifffile.imwrite(os.path.join(self.save_dir, "nucleus_color.tif"), nucleus_color_img, compression="jpeg")
        # 画细胞边界
        outline = cellpose.utils.masks_to_outlines(cells)
        nucleus_color_img[outline] = [125, 125, 125]
        cell_color_img_path = os.path.join(self.save_dir, "cell_color.tif")
        tifffile.imwrite(os.path.join(self.save_dir, "cell_color.tif"), nucleus_color_img, compression="jpeg")

        return  colors_path, nucleus_color_img_path, cell_color_img_path


class cell_to_Barcode:
    
    def __init__(self):
        pass

    @staticmethod        
    def cal_zoom_rate(width, height):
        '''计算放大倍率,返回倍率'''
        std_width = 1000
        std_height = std_width / (46 * 31) * (46 * 36 * np.sqrt(3) / 2.0)
        if std_width / std_height > width / height:
            scale = width / std_width
        else:
            scale = height / std_height
        return scale

    @staticmethod
    def barcodes_pos_with_cells(cells_path, barcode_save_dir, roi_json_path=None, filter_rate=0):
        print(f"--Start generate barcode-- roi:{roi_json_path}")
        pos_file = os.path.join(base_dir, 'barcodes_pos.tsv.gz')
        df = pd.read_csv(pos_file, compression='gzip', sep='\t', names=['name', 'pos_w', 'pos_h'], header=None)
        cells = np.load(cells_path)
        if roi_json_path.endswith(".json"):
            with open(roi_json_path, 'r') as fp:
                roi_json = json.load(fp)
            roi = roi_json["ori_group"]
            # filter_conts = np.asarray(roi)
            filter_conts = [np.asarray(r) for r in roi]
            conts_binary = np.zeros(cells.shape[:2], np.uint8)
            conts_binary = cv2.drawContours(conts_binary, filter_conts, -1, 255, -1)
            if "negative_roi_group" in roi_json:
                roi_remove = roi_json["negative_roi_group"]
                conts_binary = cv2.drawContours(conts_binary, np.asarray(roi_remove), -1, 0, -1)

            # 根据conts_binary过滤cells
            cells[conts_binary == 0] = 0

        zoom_scale = cell_to_Barcode.cal_zoom_rate(cells.shape[1], cells.shape[0])
        img_width = 1000
        std_w = 1.0 * img_width / 46 / 31 * zoom_scale
        std_h = std_w * np.sqrt(3) / 2
        radius = round(std_w * 0.618 / 2) if round(std_w * 0.618 / 2) > 1 else 1
        all_barcode = []
        count = 0
        for index, item in df.iterrows():

            #  计算barcode中心坐标位置
            tmp_w = int(item['pos_w'] * zoom_scale)
            tmp_h = int(item['pos_h'] * zoom_scale)

            #  统计此位置的barcode对应的细胞
            slice_h = slice(round(tmp_h - radius), round(tmp_h + radius))
            slice_w = slice(round(tmp_w - radius), round(tmp_w + radius))
            barcode_values = cells[slice_h, slice_w]
            #  圆形蒙版
            barcode_mask = np.zeros((radius * 2, radius * 2), dtype=np.uint8)
            cv2.circle(barcode_mask, (radius, radius), radius, 255, -1)

            calculate_values = barcode_mask.astype(bool) * barcode_values
            cell_name, pixes_num = np.unique(calculate_values, return_counts=True)
            # 计算一个barcode的面积*过滤占比,同时限制最大值不超过圆形面积，即100%
            filter_num = min(filter_rate * np.pi * np.power(radius, 2), np.sum(barcode_mask.astype(bool))-1)
            all_barcode.append([item['name'], cell_name[pixes_num > int(filter_num)]])  # 统计数目
            count += 1
            if count%10000 == 0:
                print(count)

        print("Start counting...")
        all_barcode_txt = []
        for barcode in all_barcode:
            for cell_num in barcode[-1]:
                if cell_num != 0:
                    all_barcode_txt.append("\t".join(map(str, [*barcode[:-1], f"cell_{cell_num}"])))
        print("Start saving...")
    
        with open(os.path.join(barcode_save_dir, "all_barcode_num.txt"), 'w') as f:
            f.write("\n".join(all_barcode_txt))


def masks_find_outlines(save_dir,json_name,cells_path):
    """ get outlines of masks as a 0-1 array 
    
    Parameters
    ----------------

    masks: int, 2D or 3D array 
        size [Ly x Lx] or [Lz x Ly x Lx], 0=NO masks; 1,2,...=mask labels

    Returns
    ----------------

    outlines: 2D or 3D array 
        size [Ly x Lx] or [Lz x Ly x Lx], True pixels are outlines

    """
    masks = np.load(cells_path)
    cell_contours = {}
    slices = ndi.find_objects(masks.astype(int))
    for i,si in enumerate(slices):
        if si is not None:
            sr,sc = si
            mask = (masks[sr, sc] == (i+1)).astype(np.uint8)
           #contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
            contours = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
            pvc, pvr = np.concatenate(contours[-2], axis=0).squeeze().T
            vr, vc = pvr + sr.start, pvc + sc.start
            if hasattr(vr,'__iter__'):
                
                cell_contours[str(masks[vr[0], vc[0]])] = [[int(c), int(r)] for c, r in zip(vc, vr)]
            else:
                '''
                这里会因为细胞识别会出现只有1个像素的情况，无法创建可以迭代的列表，
                1个像素太少了可以认为是误检，直接丢弃
                '''
                print()
                print(vc)
                print(vr)
                continue
    # len(cell_contours)
    json_name = json_name + '.json'
    with open(os.path.join(save_dir,json_name), "w") as fp:
        json.dumps(cell_contours)  # 先格式化，避免dump报错破坏原始文件
        json.dump(cell_contours, fp, indent=4)
    return  cell_contours
        

if __name__ == '__main__':
    print(sys.argv)
    print("---------------Cell split start-----------------")
    with open(os.path.join(base_dir, 'setup.yaml'),mode='r',encoding='utf-8') as f:
        yamlinfo = yaml.load(f,yaml.FullLoader)

    parser = argparse.ArgumentParser()
    parser.add_argument('--diameter',
                        type=str,
                        default=yamlinfo['diameter'],
                        )
    
    parser.add_argument('--seg_channel',
                        type=int,
                        default=yamlinfo['seg_channel'],
                        help="color channel")

    parser.add_argument('--fl_gray_num',
                        type=int,
                        default=0,
                        help="manual select 'fl' gray number; 0 for auto select")

    parser.add_argument('--conts_filter',
                        type=str,
                        default=yamlinfo['conts_filter'],
                        help="filter cells out of conts;'None' or 'he' or 'fl' or roi_json path")
    
    parser.add_argument('--inversion',
                        action='store_true',
                        #default=yamlinfo['inversion'],
                        help="enable inverse color")
    
    parser.add_argument('--flow_threshold',
                        default=yamlinfo['flow_threshold'],
                        type=float,
                        help='threshold to segmentation')
    
    parser.add_argument('--min_detect_value',
                        default=yamlinfo['min_detect_value'],
                        type=int,
                        help='to avoid samll pixels fault')

    parser.add_argument('--max_detect_value',
                        default=yamlinfo['max_detect_value'],
                        type=int,
                        help='nms iou thresh')
    
    parser.add_argument('--overlap',
                        default=yamlinfo['overlap'],
                        type=str,
                        help='to avoid obvious boundary lines')
    
    parser.add_argument('--sub_size',
                        default=yamlinfo['sub_size'],
                        type=str,
                        help='segemented pics size')

    parser.add_argument('--img_path',
                        type=str,
                        help='image path')

    parser.add_argument('--model_path',
                        default=yamlinfo['model_path'],
                        type=str,
                        help='model path')
    
    parser.add_argument('--json_save_dir',
                        type=str,
                        help='json save path')
    
    parser.add_argument('--json_name',
                        type=str,
                        default='outlines',
                        help='json save name')
   
    # parser.add_argument('--action',)

    parser.add_argument('--redo',
                        action='store_true',
                        #default=False,
                        help='enable redo all the seg steps')
    
    parser.add_argument('--calculate_rate',
                        type=int,
                        default=yamlinfo['calculate_rate'],
                        help='rate to speed up computation')
    
    parser.add_argument('--cell_edge',
                        type=int,
                        default=yamlinfo['cell_edge'],
                        help='cell boundaries extend pixel value')
    
    parser.add_argument('--find_outlines',
                        type=str,
                        default=None,
                        help='if you want to find outlines of a specific segged \
                        nucleus npy or cells npy, use this command with its file path\
                        如果传入确定的npy路径参数,那么跳过所有分割步骤;如果未传入路径\
                        则默认执行前三步分割,以cells.npy路径为参数传入')
   
    parser.add_argument('--generate_barcode',
                        type=str,
                        default=None,
                        help='只能传入cells.npy 路径\
                        如果传入确定的npy路径参数,那么跳过所有分割步骤;生成allbarcode.txt文件')    
    
    parser.add_argument('--barcode_save_dir',
                        type=str,
                        default=None,
                        help='barcode保存路径')    
    
    parser.add_argument('--save_dir',
                        type=str,
                        default=None,
                        help='过程文件保存路径')

    parser.add_argument('--nucleus_path',
                        type=str,
                        default=None,
                        help='预处理细胞核（细胞）矩阵路径，后续将基于此矩阵继续分割')

    parser.add_argument('--enhance',
                        action='store_true',
                        #default=False,
                        help='如果分割图像梯度不足导致分割效果差，可以加上此参数，重新分割')

    parser.add_argument('--barcode_filter_rate',
                        type=float,
                        default=0,
                        help='（0-1）,给定barcode与细胞接触比率，默认为0：barcode与细胞有碰撞即记录，0.5：barcode需要有>50%与细胞碰撞。。')
    parser.add_argument('--expand_boundary',
                        type=str,
                        default=None,
                        help="单独执行分水岭算法扩充边界，接字符串参数，不同参数按顺序用分号分割，如："
                             "'nucleus_path;cell_edge;save_dir;cells_npy_name'")
    args = parser.parse_args()

    # 只做核边界扩充
    if args.expand_boundary is not None:
        print(args.expand_boundary)
        nucleus_path, cell_edge, save_dir, cells_npy_name = args.expand_boundary.split(";")
        cell_edge = int(cell_edge)
        print(nucleus_path, cell_edge, save_dir, cells_npy_name)
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)

        dc = DrawCell(cell_edge=cell_edge, calculate_rate=args.calculate_rate, nucleus_path=nucleus_path,
                      save_dir=save_dir, cells_npy=cells_npy_name)
        masks, cells, cells_path = dc.generate_cells()
        colors_path, nucleus_color_img_path, cell_color_img_path = dc.cells_coloring(masks=masks, cells=cells)

        sys.exit(0)


    find_outlines_path = None
############False###################False#####################
    if not (args.find_outlines or args.generate_barcode):
        # find_outlines,barcode有一个传入参数，则跳过前两步 
        ##############1######################
        segModel = CellSeg(save_dir = args.save_dir,diameter=eval(args.diameter),seg_channel = args.seg_channel,
                           inversion = args.inversion,flow_threshold = args.flow_threshold, conts_filter=args.conts_filter,
                           min_detect_value = args.min_detect_value,max_detect_value = args.max_detect_value,
                           sub_size = eval(args.sub_size), overlap = eval(args.overlap), img_path = args.img_path,
                           model_path=args.model_path, nucleus_base_path=args.nucleus_path, enhance=args.enhance)

        model = segModel.create_model()
        img, cell_mask =  segModel.load_image()

        '''分割子图前判断是否存nucleus.npy'''
        print(args.img_path)
        nucleus_path = os.path.join(segModel.save_dir,'nucleus.npy')
        if os.path.exists(nucleus_path) and (not args.redo):  
            ''' npy存在True  重做 True ---> False
                npy不存在False 不重做 False ---> False
                npy存在True  不重做 True ---> True
                npy不存在False 重做 True ---> False
            '''
            print('nucleus.npy already exits!')
            print('nucleus.npy saved in ',nucleus_path)
        else:
            segged_img = segModel.seg_image(img = img,cell_mask = cell_mask,model = model)
            nucleus_path = segModel.splice_masks(img = segged_img)





        ##############2######################
        dc = DrawCell(cell_edge = args.cell_edge,calculate_rate = args.calculate_rate,nucleus_path = nucleus_path)
        '''分割子图前判断是否存cells.npy'''
        cells_path = os.path.join(segModel.save_dir,'cells.npy')
        if os.path.exists(cells_path) and (not args.redo):
            print('cells.npy already exits!')
            print(nucleus_path)
            masks = np.load(nucleus_path)
            cells = np.load(cells_path)
        else:
            masks, cells, cells_path = dc.generate_cells()

        colors_path, nucleus_color_img_path, cell_color_img_path = dc.cells_coloring(masks = masks,cells = cells)
            
        
        
     
        #cells_path = cells_path


    if  (args.generate_barcode or not args.find_outlines):
        ##############3######################
        ### barcode 要单独修改
        cells2barcode_path = args.generate_barcode
        barcode_save_dir = args.barcode_save_dir
        if barcode_save_dir and not os.path.exists(barcode_save_dir):
            os.mkdir(barcode_save_dir)

        ## 
        if cells2barcode_path and barcode_save_dir:
            #cb = cell_to_Barcode(cells_path=cells2barcode_path)
            cell_to_Barcode.barcodes_pos_with_cells(cells_path=cells2barcode_path,
                                                    barcode_save_dir=barcode_save_dir,
                                                    roi_json_path=args.conts_filter,
                                                    filter_rate=args.barcode_filter_rate)

        else:
            cells2barcode_path = cells_path
            barcode_save_dir = os.path.split(cells_path)[0]
            print('barcode save dir is changed to cells path')
        print(f"barcode_save_dir:{barcode_save_dir}; args.redo:{args.redo}")
        if os.path.exists(os.path.join(barcode_save_dir, "all_barcode_num.txt")) and not args.redo:
            print('all_barcode_num.txt exists and saved in you assigned path!')
        else:
            #cb = cell_to_Barcode(cells_path=cells_path,barcode_save_dir=args.barcode_save_dir)
            cell_to_Barcode.barcodes_pos_with_cells(cells_path=cells2barcode_path, barcode_save_dir=barcode_save_dir,
                                                    roi_json_path=args.conts_filter,
                                                    filter_rate=args.barcode_filter_rate)

        #find_outlines_path = cells_path

    ##############4######################
    '''
    find_outlines 与 generate_barcode都为空运行默认生成 不接受指定参数,    F F T
    find_outlines 不为空， barcode为空运行指定生成 ，接受指定参数,         T F T
    find_outlines为空， barcode不为空 不运行                             F T F
    二者都不为空，运行                                                   T T T
      '''
    if  (args.find_outlines or not args.generate_barcode):

        t1 = time.time()

        #初始化参数
        find_outlines_path = args.find_outlines
        json_save_dir = args.json_save_dir
        json_name = args.json_name


        # 三者同时指定，按照参数运行
        if json_save_dir and find_outlines_path and json_name:
            masks_find_outlines(json_save_dir,json_name,cells_path=find_outlines_path)
        else:
            #默认生成两个

            if args.save_dir:
                json_save_dir = args.save_dir
            else:
                json_save_dir  = os.path.splitext(args.img_path)[0]
            #生成cells
            masks_find_outlines(json_save_dir,json_name='cellConts',cells_path=cells_path)

            #生成nucleus
            masks_find_outlines(json_save_dir,json_name='nucleusConts',cells_path=nucleus_path)

        t2 = time.time()
        print('outline time occupied',t2 - t1)



        
