import os
import gzip
import numpy as np
import pandas as pd

from optparse import OptionParser

parser=OptionParser(description='Generates matrix files')
parser.add_option('-i', '--input', '--in', default='.', help="BSTViewer_project/level_matrix/level_1/ [default: %default]")
parser.add_option('-c', default='all_barcode_num.txt', help="level1 barcode id and cell name mapping file")
parser.add_option('-o', '--output', '--out', default='.', help="where to output the gene count matrix [default: %default]")
(opts, args)=parser.parse_args()

print(opts)
if not os.path.exists(opts.output):
        os.mkdir(opts.output)
#barcode_file = "/share/nas1/dengdj/project/spatial/20230303_B052-1-2-3-4/YF-ZXY-GW-0102-20230103/analysis/05.AllheStat/BSTViewer_project/level_matrix/level_1/barcodes.tsv.gz"
barcode_file = f'{opts.input}/barcodes.tsv.gz'
#matrix_file = "/share/nas1/dengdj/project/spatial/20230303_B052-1-2-3-4/YF-ZXY-GW-0102-20230103/analysis/05.AllheStat/BSTViewer_project/level_matrix/level_1/matrix.mtx.gz"
matrix_file = f"{opts.input}/matrix.mtx.gz"
#feature_file = "/share/nas1/dengdj/project/spatial/20230303_B052-1-2-3-4/YF-ZXY-GW-0102-20230103/analysis/05.AllheStat/BSTViewer_project/level_matrix/level_1/features.tsv.gz"
feature_file = f"{opts.input}/features.tsv.gz"

#cell_df = pd.read_csv("YF-20230103-ZXY-GW-0102/all_barcode_num.txt", sep='\t', names=["barcode_id","cell"],header=None)
cell_df = pd.read_csv(opts.c, sep='\t', names=["barcode_id","cell"],header=None)
cell=pd.DataFrame(cell_df['cell'].unique())
cell=cell.rename(columns={0:"cell"})
#cell.to_csv("barcodes.tsv.gz",compression="gzip",sep="\t",header=False,index=False)
cell.to_csv(f"{opts.output}/barcodes.tsv.gz",compression="gzip",sep="\t",header=False,index=False)
cell['cell_line'] = range(1, 1 + cell.shape[0])

df = pd.read_csv(matrix_file, compression='gzip', sep='\t', names=['gene', 'supspot', 'umi'], skiprows=3,header=None)
feature_df = pd.read_csv(feature_file, compression='gzip', sep='\t', header=None)
#feature_df.to_csv("features.tsv.gz",compression="gzip",sep="\t",header=False,index=False)
feature_df.to_csv(f"{opts.output}/features.tsv.gz",compression="gzip",sep="\t",header=False,index=False)
barcode_df = pd.read_csv(barcode_file, compression='gzip', sep='\t', names=['barcode_id'],header=None)
barcode_df['supspot'] = range(1, 1 + barcode_df.shape[0])
df=pd.merge(df,barcode_df,on="supspot")
df=pd.merge(df,cell_df,on="barcode_id")

umi_num = df.groupby(['gene','cell'], as_index=False).agg({'umi': 'sum'}).copy()
#print(df.head())
umi_num = pd.merge(umi_num,cell,on="cell")
umi_num = umi_num[['gene','cell_line','umi']]
head=f"%%MatrixMarket matrix coordinate real general\n%\n{feature_df.shape[0]}\t{cell.shape[0]}\t{umi_num.shape[0]}\n"
with gzip.open(f'{opts.output}/matrix.mtx.gz','wt') as f:
    f.write(head)
umi_num.to_csv(f"{opts.output}/matrix.mtx.gz",compression="gzip",sep="\t",header=False,index=False,mode="a")


supspot_num = umi_num['cell_line'].value_counts().shape[0]
median_gene_num = umi_num['cell_line'].value_counts().median()
median_umi_num = umi_num['umi'].astype(float).groupby(umi_num['cell_line']).sum().median()
total_gene_num = umi_num['gene'].value_counts().shape[0]

with open(f"{opts.output}/stat.xls",'w') as fp:
    #fp.write(f"cell_num\t{supspot_num}\ntotal_gene_num\t{total_gene_num}\nmedian_gene_num\t{median_gene_num}\nmedian_umi_num\t{median_umi_num}\n")
    fp.write(f"cell_num\t{supspot_num}\nmedian_gene_num\t{median_gene_num}\nmedian_umi_num\t{median_umi_num}\ntotal_gene_num\t{total_gene_num}\n")

