#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2023
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2023
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2023
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","o=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           config.txt file                    <infile>     must be given
		-o           outdir                             [outdir]     optional [OUTDIR/backup/]
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $cfgfile = $opts{i} ;

## reading cfg file
my %hcfg = ();
&reading_cfg_file($cfgfile, \%hcfg);

## 
my $analysis_dir = $hcfg{OUTDIR} ;
my $prefix = $hcfg{PREFIX} ;
my $outdir = defined $opts{o} ? $opts{o} : "$analysis_dir/backup" ;
mkdir $outdir if (!-d $outdir);

my $cmd = "cp $cfgfile $outdir/" ;
&run_or_die($cmd) ;
mkdir "$outdir/01.fastq2BcUmi" if (!-d "$outdir/01.fastq2BcUmi") ;
$cmd = "cp $analysis_dir/01.fastq2BcUmi/$prefix.bc_dist $analysis_dir/01.fastq2BcUmi/$prefix.bc_stat $analysis_dir/01.fastq2BcUmi/$prefix.full_stat $analysis_dir/01.fastq2BcUmi/$prefix.qual.stat $outdir/01.fastq2BcUmi/" ;
&run_or_die($cmd) ;

if (-f "$analysis_dir/02.LinkBcChip/$prefix.flu.stat"){
    mkdir "$outdir/02.LinkBcChip/" if (!-d "$outdir/02.LinkBcChip/");
    $cmd = "cp $analysis_dir/02.LinkBcChip/$prefix.flu.stat $outdir/02.LinkBcChip/" ;
    &run_or_die($cmd) ;
}

mkdir "$outdir/03.Umi2Gene" if (!-d "$outdir/03.Umi2Gene") ;
$cmd = "cp $analysis_dir/03.Umi2Gene/$prefix"."Log.final.out $analysis_dir/03.Umi2Gene/$prefix.total.stat $outdir/03.Umi2Gene/" ;
&run_or_die($cmd) ;

if (-f "$analysis_dir/04.MatrixMake/$prefix.sequencing_saturation.stat"){
    mkdir "$outdir/04.MatrixMake" if (!-d "$outdir/04.MatrixMake");
    $cmd = "cp $analysis_dir/04.MatrixMake/$prefix.sequencing_saturation.stat $outdir/04.MatrixMake/" ;
    &run_or_die($cmd) ;
}
if (-f "$analysis_dir/04.MatrixMake/$prefix.sequencing_saturation.png"){
    mkdir "$outdir/04.MatrixMake" if (!-d "$outdir/04.MatrixMake");
    $cmd = "cp $analysis_dir/04.MatrixMake/$prefix.sequencing_saturation.png $outdir/04.MatrixMake/" ;
    &run_or_die($cmd) ;
}

mkdir "$outdir/05.AllheStat/" if (!-d "$outdir/05.AllheStat/") ;
$cmd = "cp -r $analysis_dir/05.AllheStat/BSTViewer_project/ $analysis_dir/05.AllheStat/stat.txt $analysis_dir/05.AllheStat/all_level_stat.txt $outdir/05.AllheStat/" ;
&run_or_die($cmd) ;
mkdir "$outdir/05.AllheStat/allhe/" if (!-d "$outdir/05.AllheStat/allhe/") ;
$cmd = "cp $analysis_dir/05.AllheStat/allhe/he_roi_small.png $outdir/05.AllheStat/allhe/" ;
&run_or_die($cmd) ;
mkdir "$outdir/05.AllheStat/umi_plot/" if (!-d "$outdir/05.AllheStat/umi_plot/") ;
$cmd = "cp $analysis_dir/05.AllheStat/umi_plot/roi_umi_count_small.png $analysis_dir/05.AllheStat/umi_plot/all_umi_count_small.png $outdir/05.AllheStat/umi_plot/" ;
&run_or_die($cmd) ;

mkdir "$outdir/06.Cluster/" if (!-d "$outdir/06.Cluster/") ;
mkdir "$outdir/06.Cluster/L13/" ;
$cmd = "cp $analysis_dir/06.Cluster/L13/*.* $outdir/06.Cluster/L13/" ;
&run_or_die($cmd) ;
mkdir "$outdir/06.Cluster/L3/" ;
$cmd = "cp $analysis_dir/06.Cluster/L3/*.* $outdir/06.Cluster/L3/" ;
&run_or_die($cmd) ;
mkdir "$outdir/06.Cluster/L4/" ;
$cmd = "cp $analysis_dir/06.Cluster/L4/*.* $outdir/06.Cluster/L4/" ;
&run_or_die($cmd) ;
mkdir "$outdir/06.Cluster/L5/" ;
$cmd = "cp $analysis_dir/06.Cluster/L5/*.* $outdir/06.Cluster/L5/" ;
&run_or_die($cmd) ;
mkdir "$outdir/06.Cluster/L6/" ;
$cmd = "cp $analysis_dir/06.Cluster/L6/*.* $outdir/06.Cluster/L6/" ;
&run_or_die($cmd) ;
mkdir "$outdir/06.Cluster/L7/" ;
$cmd = "cp $analysis_dir/06.Cluster/L7/*.* $outdir/06.Cluster/L7/" ;
&run_or_die($cmd) ;

if (-d "$analysis_dir/07.CellSplit/"){
    mkdir "$outdir/07.CellSplit/" if (!-d "$outdir/07.CellSplit/") ;
    mkdir "$outdir/07.CellSplit/cell_split_result/" if (!-d "$outdir/07.CellSplit/cell_split_result/") ;
    if (-f "$analysis_dir/07.CellSplit/cell_split_result/nucleus.npy" && -f "$analysis_dir/07.CellSplit/cell_split_result/cells.npy" && -f "$analysis_dir/07.CellSplit/cell_split_result/colors.npy"){ 
        $cmd = "cp $analysis_dir/07.CellSplit/cell_split_result/nucleus.npy $analysis_dir/07.CellSplit/cell_split_result/cells.npy $analysis_dir/07.CellSplit/cell_split_result/colors.npy $outdir/07.CellSplit/cell_split_result/" ;
        &run_or_die($cmd) ;
    }
    if (-f "$analysis_dir/07.CellSplit/cell_split_result/fluorescence.tif" && -f "$analysis_dir/07.CellSplit/cell_split_result/all_barcode_num.txt"){
        $cmd = "cp $analysis_dir/07.CellSplit/cell_split_result/fluorescence.tif $analysis_dir/07.CellSplit/cell_split_result/all_barcode_num.txt $outdir/07.CellSplit/cell_split_result/" ;
        &run_or_die($cmd) ;
    }
    
    $cmd = "cp -r $analysis_dir/07.CellSplit/mtx/ $analysis_dir/07.CellSplit/images/ $analysis_dir/07.CellSplit/cluster/ $outdir/07.CellSplit/" ;
    &run_or_die($cmd) ;
}

$cmd = "cp -r $analysis_dir/0?.WebReport/ $analysis_dir/$prefix/ $outdir/" ;
&run_or_die($cmd) ;

if (-f "$analysis_dir/readme.txt"){
    $cmd = "cp $analysis_dir/readme.txt $outdir/" ;
    &run_or_die($cmd) ;
}

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&reading_cfg_file($cfgfile, \%hcfg);
sub reading_cfg_file()
{
    my ($cfgfile, $ahcfg) = @_ ;

    open (IN, $cfgfile) || die "$cfgfile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        if (m/^STAR_para\s+(.*)/){
            $ahcfg->{STAR_para} = $1 ;
            $ahcfg->{STAR_para} =~ s/\s*$// ;
        }
        else{
            my ($key, $value) = split ;
            $ahcfg->{$key} = $value ;
        }
    }
    close(IN);

    return ;
}

