#!/usr/bin/perl -w
use strict ;
use File::Basename qw(basename dirname);

if (@ARGV != 2){
    print "Usage: perl $0 configfile outfile\n" ;
    exit(1);
}

my ($infile, $outfile) = @ARGV ;

open (IN, $infile) || die "$infile, $!\n" ;
my %hcfg= ();
while(<IN>){
    chomp ;
    next if (m/^\#|^\s*$/);
    my ($key, $value) = split ;
    $hcfg{$key} = $value ;
}
close(IN);

open (OUT, ">$outfile") || die "$outfile, $!\n" ;
print OUT "FullBC(%)\tFraction(%)\tUMI(M)\tSaturation/MedianGene(L13)\n" ;
my $prefix = $hcfg{PREFIX} ;
my $dir = $hcfg{OUTDIR} ;

my $bc_ratio = "-" ;
my $bcfile = "$dir/01.fastq2BcUmi/$prefix.bc_stat" ;
if (-f $bcfile){
    open (IN, $bcfile) || die "$bcfile, $!\n" ;
    while(<IN>){
        chomp ;
        if (m/^bc1-bc2-bc3/){
            $bc_ratio = (split /\s+/)[2] ;
        }
    }
    close(IN);
}
print OUT "$bc_ratio" ;

my $frac = "-" ;
my $saturat = "-" ;
my $spot_stat_file = "$dir/05.AllheStat/stat.txt" ;
if (-f $spot_stat_file){
    open (IN, $spot_stat_file) || die "$spot_stat_file, $!\n" ;
    while(<IN>){
        chomp ;
        if (m/sequencing_saturation\s+(\d*\.?\d+\%)/){
            $saturat = $1 ;
        }
        elsif (m/fraction_reads_in_spots_under_tissue\s+(\d*\.?\d+\%)/){
            $frac = $1 ;
        }
    }
    close(IN);
}
print OUT "\t$frac" ;

my $umi_num = "-" ;
my $bc_dist = "$dir/01.fastq2BcUmi/$prefix.bc_dist" ;
if (-f $bc_dist){
    open (IN, $bc_dist) || die "$bc_dist, $!\n" ;
    my $sum = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my $umi = (split)[3] ;
        $sum += $umi ;
    }
    close(IN);
    $umi_num = $sum/3 ;
}
if ($umi_num ne '-'){
    $umi_num = int($umi_num/1000000*100+0.5)/100 ;
}
print OUT "\t$umi_num" ;


my $median_gene = "-" ;
my $all_level_stat = "$dir/05.AllheStat/all_level_stat.txt" ;
if (-f $all_level_stat){
    open (IN, $all_level_stat) || die "$all_level_stat, $!\n" ;
    while(<IN>){
        chomp ;
        if (m/median_gene_num/){
            $median_gene = (split)[1] ;
        }
    }
    close(IN);
}
print OUT "\t$saturat/$median_gene\n" ;
close(OUT);

