#!/usr/bin/perl -w
use strict ;
use File::Basename qw(basename dirname);

if (@ARGV != 2){
    print "Usage: perl $0 filelist outfile\n" ;
    exit(1);
}

my ($infile, $outfile) = @ARGV ;

open (IN, $infile) || die "$infile, $!\n" ;
my %hfile = ();
while(<IN>){
    chomp ;
    next if (m/^\#|^\s*$/);
    my ($id, $file) = split ;
    $hfile{$id} = $file ;
}
close(IN);

open (OUT, ">$outfile") || die "$outfile, $!\n" ;
print OUT "#Sample\tData(G)\tFraction\tSaturation\tBCratio\tTotalMapped\tConfMapped\tIntergenicMap\tIntronicMap\tExonicMap\tTranscriptomeMap\t100um\t50um\t42um\t35um\t27um\t20um\t10um\t5um\n" ;
my $basename = basename($infile) ;
$basename =~ s/.filelist$// ;
print OUT "$basename" ;
my $data = "-" ;
my $bc_ratio = "-" ;
if (defined $hfile{QUAL}){
    open (IN, $hfile{QUAL}) || die "$hfile{QUAL}, $!\n" ;
    while(<IN>){
        chomp ;
        if (m/Number of Reads\s+(\d+)/){
            my $reads = $1 ;
            $data = int(($reads * 300)/1000000000+0.5) ;
        }
        elsif (m/Final Valid Reads\s+\d+\s+(\d*\.?\d+)/){
            $bc_ratio = $1 ;
        }
    }
    close(IN);
}
my $frac = "-" ;
my $saturat = "-" ;
if (defined $hfile{SPOT}){
    open (IN, $hfile{SPOT}) || die "$hfile{SPOT}, $!\n" ;
    while(<IN>){
        chomp ;
        if (m/sequencing_saturation\s+(\d*\.?\d+\%)/){
            $saturat = $1 ;
        }
        elsif (m/fraction_reads_in_spots_under_tissue\s+(\d*\.?\d+\%)/){
            $frac = $1 ;
        }
    }
    close(IN);
}
print OUT "\t$data\t$frac\t$saturat\t$bc_ratio" ;
my $totalmap = "-" ;
my $confmap = "-" ;
my $Intergenicmap = "-" ;
my $Intronicmap = "-" ;
my $Exonicmap = "-" ;
my $Transcriptomemap = "-" ;
if (defined $hfile{MAP}){
    open (IN, $hfile{MAP}) || die "$hfile{MAP}, $!\n" ;
    while(<IN>){
        chomp ;
        if (m/Reads Mapped to Genome\s+\d+\s+(\d*\.?\d+\%)/){
            $totalmap = $1 ;
        }
        elsif (m/Reads Mapped Confidently to Genome\s+\d+\s+(\d*\.?\d+\%)/){
            $confmap = $1 ;
        }
        elsif (m/Reads Mapped Confidently to Intergenic Regions\s+\d+\s+(\d*\.?\d+\%)/){
            $Intergenicmap = $1 ;
        }
        elsif (m/Reads Mapped Confidently to Intronic Regions\s+\d+\s+(\d*\.?\d+\%)/){
            $Intronicmap = $1 ;
        }
        elsif (m/Reads Mapped Confidently to Exonic Regions\s+\d+\s+(\d*\.?\d+\%)/){
            $Exonicmap = $1 ;
        }
        elsif (m/Reads Mapped Confidently to Transcriptome\s+\d+\s+(\d*\.?\d+\%)/){
            $Transcriptomemap = $1 ;
        }
    }
    close(IN);
}
print OUT "\t$totalmap\t$confmap\t$Intergenicmap\t$Intronicmap\t$Exonicmap\t$Transcriptomemap" ;
if (defined $hfile{LEVEL}){
    open (IN, $hfile{LEVEL}) || die "$hfile{LEVEL}, $!\n" ;
    while(<IN>){
        chomp ;
        if (m/median_gene_num/){
            my $genes = (split/\s+/, $_, 2)[1] ;
            print OUT "\t$genes\n" ;
        }
    }
    close(IN);
}
else{
    print OUT "\t-\t-\t-\t-\t-\t-\t-\n" ;
}

close(OUT);

