#!/usr/bin/perl -w
use strict ;

if (@ARGV < 4){
    print "Usage: perl $0 infile selectfile outfile cutlen [start] [type:B/E]\n" ;
    exit(1);
}

my ($infile, $selectfile, $outfile, $cutlen, $start, $type) = @ARGV ;
$start ||= 0 ;
$type ||= "B" ;

open (IN, $selectfile) || die "$selectfile, $!\n" ;
my @ids = ();
while(<IN>){
    chomp ;
    next if (m/^\#|^\s*$/);
    my ($id) = split ;
    $ids[$id] = 1 ;
}
close(IN);

if ($infile =~ /\.fa$|\.fasta$/i){
    open (IN, $infile) || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    $/ = ">" ;
    my $r_index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$/);
        if (!defined $ids[$r_index]){
            $r_index++ ;
            next ;
        }
        my $r_id = $r_index ;
        $r_index++ ;
        my ($id, $seq) = split /\n/, $_, 2 ;
        $seq =~ s/\n//g ;
        my $len = length($seq) ;
        if ($type eq 'E'){
            my $new_seq = substr($seq, $len-$cutlen-$start, $cutlen);
            print OUT ">$r_id\n$new_seq\n" ;
        }
        else{
            my $new_seq = substr($seq, $start, $cutlen);
            print OUT ">$r_id\n$new_seq\n" ;
        }
    }
    $/ = "\n" ;
    close(IN);
    close(OUT);
}
elsif($infile =~ /\.fq$|\.fastq$/i){
    open (IN, $infile) || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my $r_index = 0 ;
    while(<IN>){
        my $id = $_ ;
        my $seq = <IN> ;
        my $id2 = <IN> ;
        my $qual = <IN> ;
        chomp($id, $seq, $id2, $qual);
        if (!defined $ids[$r_index]){
            $r_index++ ;
            next ;
        }
        my $r_id = $r_index ;
        $r_index++ ;
        my $len = length($seq) ;
        if ($type eq 'E'){
            my $new_seq = substr($seq, $len-$cutlen-$start, $cutlen);
            my $new_qual = substr($qual, $len-$cutlen-$start, $cutlen);
            print OUT "\@$r_id\n$new_seq\n\+\n$new_qual\n" ;
        }
        else{
            my $new_seq = substr($seq, $start, $cutlen);
            my $new_qual = substr($qual, $start, $cutlen) ;
            print OUT "\@$r_id\n$new_seq\n\+\n$new_qual\n" ;
        }
    }
    close(IN);
    close(OUT);
}
elsif($infile =~ /\.fq.gz$|\.fastq.gz$/i){
    open (IN, "gunzip -c $infile |") || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my $r_index = 0 ;
    while(<IN>){
        my $id = $_ ;
        my $seq = <IN> ;
        my $id2 = <IN> ;
        my $qual = <IN> ;
        chomp($id, $seq, $id2, $qual);
        if (!defined $ids[$r_index]){
            $r_index++ ;
            next ;
        }
        my $r_id = $r_index ;
        $r_index++ ;
        my $len = length($seq) ;
        if ($type eq 'E'){
            my $new_seq = substr($seq, $len-$cutlen-$start, $cutlen);
            my $new_qual = substr($qual, $len-$cutlen-$start, $cutlen);
            print OUT "\@$r_id\n$new_seq\n\+\n$new_qual\n" ;
        }
        else{
            my $new_seq = substr($seq, $start, $cutlen);
            my $new_qual = substr($qual, $start, $cutlen) ;
            print OUT "\@$r_id\n$new_seq\n\+\n$new_qual\n" ;
        }
    }
    close(IN);
    close(OUT);
}


