#!/usr/bin/perl -w
use strict ;

if (@ARGV < 3){
    print "Usage: perl $0 infile outfile cutlen [start] [type:B/E]\n" ;
    exit(1);
}

my ($infile, $outfile, $cutlen, $start, $type) = @ARGV ;
$start ||= 0 ;
$type ||= "B" ;

if ($infile =~ /\.fa$|\.fasta$/i){
    open (IN, $infile) || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    $/ = ">" ;
    my $r_index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$/);
        my ($id, $seq) = split /\n/, $_, 2 ;
        $seq =~ s/\n//g ;
        my $len = length($seq) ;
        if ($type eq 'E'){
            if ($cutlen > $len - $start){
                my $new_seq = substr($seq, 0, $len-$start);
                print OUT ">$r_index\n$new_seq\n" ;
            }
            else{
                my $new_seq = substr($seq, $len-$cutlen-$start, $cutlen);
                print OUT ">$r_index\n$new_seq\n" ;
            }
        }
        else{
            if ($cutlen > $len - $start){
                my $new_seq = substr($seq, $start);
                print OUT ">$r_index\n$new_seq\n" ;
            }
            else{
                my $new_seq = substr($seq, $start, $cutlen);
                print OUT ">$r_index\n$new_seq\n" ;
            }
        }
        $r_index++ ;
    }
    $/ = "\n" ;
    close(IN);
    close(OUT);
}
elsif($infile =~ /\.fq$|\.fastq$/i){
    open (IN, $infile) || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my $r_index = 0 ;
    while(<IN>){
        my $id = $_ ;
        my $seq = <IN> ;
        my $id2 = <IN> ;
        my $qual = <IN> ;
        chomp($id, $seq, $id2, $qual);
        my $len = length($seq) ;
        if ($type eq 'E'){
            if ($cutlen > $len-$start){
                my $new_seq = substr($seq, 0, $len-$start);
                my $new_qual = substr($qual, 0, $len-$start);
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
            else{
                my $new_seq = substr($seq, $len-$cutlen-$start, $cutlen);
                my $new_qual = substr($qual, $len-$cutlen-$start, $cutlen);
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
        }
        else{
            if ($cutlen > $len - $start){
                my $new_seq = substr($seq, $start);
                my $new_qual = substr($qual, $start) ;
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
            else{
                my $new_seq = substr($seq, $start, $cutlen);
                my $new_qual = substr($qual, $start, $cutlen) ;
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
        }
        $r_index++ ;
    }
    close(IN);
    close(OUT);
}
elsif($infile =~ /\.fq.gz$|\.fastq.gz$/i){
    open (IN, "gunzip -c $infile |") || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my $r_index = 0 ;
    while(<IN>){
        my $id = $_ ;
        my $seq = <IN> ;
        my $id2 = <IN> ;
        my $qual = <IN> ;
        chomp($id, $seq, $id2, $qual);
        my $len = length($seq) ;
        if ($type eq 'E'){
            if ($cutlen > $len - $start){
                my $new_seq = substr($seq, 0, $len-$start);
                my $new_qual = substr($qual, 0, $len-$start);
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
            else{
                my $new_seq = substr($seq, $len-$cutlen-$start, $cutlen);
                my $new_qual = substr($qual, $len-$cutlen-$start, $cutlen);
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
        }
        else{
            if ($cutlen > $len - $start){
                my $new_seq = substr($seq, $start);
                my $new_qual = substr($qual, $start) ;
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
            else{
                my $new_seq = substr($seq, $start, $cutlen);
                my $new_qual = substr($qual, $start, $cutlen) ;
                print OUT "\@$r_index\n$new_seq\n\+\n$new_qual\n" ;
            }
        }
        $r_index++ ;
    }
    close(IN);
    close(OUT);
}


