#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"u=s","b=s","g=s","o=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{u}) || !defined($opts{b}) || !defined($opts{g}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-u           bc umi reads file                      <infile>     must be given
		-b           barcode file                           <infile>     must be given
		-g           umi gene file                          <infile>     must be given
		-o           result file                            <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $umifile = $opts{u} ;
my $bcfile = $opts{b} ;
my $ugfile = $opts{g} ;
my $outfile = $opts{o} ;

## get final barcode
my %hbc = ();
&get_barcode($bcfile, \%hbc);

## get umi gene
my %humi = ();
&get_umi_gene(\%hbc, $ugfile, \%humi);

## get bc umi and stat
&get_bc_umi_and_stat($umifile, \%hbc, \%humi, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_barcode($bcfile, \%hbc);
sub get_barcode()
{
    my ($bcfile, $ahbc) = @_ ;

    open (IN, $bcfile) || die "$bcfile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($bc) = split ;
        $ahbc->{$bc} = 1 ;
    }
    close(IN);

    return ;
}

#&get_umi_gene(\%hbc, $ugfile, \%humi);
sub get_umi_gene()
{
    my ($ahbc, $ugfile, $ahumi) = @_ ;

    open (IN, $ugfile) || die "$ugfile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($bc, $umi, $gene) = split ;
        next if (!defined $ahbc->{$bc}) ;
        $ahumi->{$bc}{$umi}++ ;
    }
    close(IN);

    return ;
}

#&get_bc_umi_and_stat($umifile, \%hbc, \%humi, $outfile);
sub get_bc_umi_and_stat()
{
    my ($umifile, $ahbc, $ahumi, $outfile) = @_ ;

    open (IN, $umifile) || die "$umifile, $!\n" ;
    my %hraw_bc = ();
    my $raw_rdnum = 0 ;
    my $raw_umi_num = 0 ;
    my $new_rdnum = 0 ;
    my $new_umi_num = 0 ;
    my $new2_rdnum = 0 ;
    my $new2_umi_num = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($bc, $umi, $rdnum) = split ;
        $hraw_bc{$bc}++ ;
        $raw_rdnum += $rdnum ;
        $raw_umi_num++ ;
        if (defined $ahbc->{$bc}){
            $new_rdnum += $rdnum ;
            $new_umi_num++ ;
        }
        if (defined $ahumi->{$bc} && defined $ahumi->{$bc}{$umi}){
            $new2_rdnum += $rdnum ;
            $new2_umi_num++ ;
        }
    }
    close(IN);

    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my $raw_bc_num = scalar(keys %hraw_bc) ;
    my $new_bc_num = scalar(keys %{$ahbc}) ;
    my $new2_bc_num = scalar(keys %{$ahumi}) ;
    print OUT "#Type\tBCNumber\tUMINumber\tReadsNumber\n" ;
    print OUT "01.fastq2BcUmi\t$raw_bc_num\t$raw_umi_num\t$raw_rdnum\n" ;
    print OUT "02.LinkBcChip\t$new_bc_num\t$new_umi_num\t$new_rdnum\n" ;
    print OUT "03.Umi2Gene\t$new2_bc_num\t$new2_umi_num\t$new2_rdnum\n" ;
    close(OUT);

    return ;
}

