# -*- coding: UTF-8 -*-
import os
import getopt
import sys
import pandas as pd


# 统计supspot数量，中位基因数，中位UMI数
def median_num_stat(matrix_file, skiprows=3):
    df = pd.read_csv(matrix_file, compression='gzip', sep='\t', names=['gene', 'supspot', 'umi'], skiprows=skiprows,
                     header=None)

    supspot_num = df['supspot'].value_counts().shape[0]
    q40_gene_num = df['supspot'].value_counts().quantile(0.4)
    median_gene_num = df['supspot'].value_counts().median()
    q60_gene_num = df['supspot'].value_counts().quantile(0.6)
    q40_umi_num = df['umi'].astype(float).groupby(df['supspot']).sum().quantile(0.4)
    median_umi_num = df['umi'].astype(float).groupby(df['supspot']).sum().median()
    q60_umi_num = df['umi'].astype(float).groupby(df['supspot']).sum().quantile(0.6)
    total_gene_num = df['gene'].value_counts().shape[0]
    return([supspot_num, q40_gene_num, median_gene_num, q60_gene_num, q40_umi_num, median_umi_num, q60_umi_num, total_gene_num])
    pass

def stat_all_level_info(used_args):
    # init
    matrix_dir = used_args['matrix_dir']
    levels = used_args['levels'].split(',')
    df = pd.DataFrame()
    name = ['level', 'supspot_num', 'q40_gene_num', 'median_gene_num', 'q60_gene_num', 'q40_umi_num', 'median_umi_num', 'q60_umi_num', 'total_gene_num']
    df['name'] = name
    # all level stat
    for level in levels:
        matrix_file = f"{matrix_dir}/subdata/L{level}_heAuto/matrix.mtx.gz"
        supspot_num, q40_gene_num, median_gene_num, q60_gene_num, q40_umi_num, median_umi_num, q60_umi_num, total_gene_num = median_num_stat(matrix_file=matrix_file)
        curr_level = f"level{level}"
        df[curr_level] = [int(level), supspot_num, q40_gene_num, median_gene_num, q60_gene_num, q40_umi_num, median_umi_num, q60_umi_num, total_gene_num]
    # output
    content = df.to_csv(sep='\t', header=False, index=False, lineterminator='\n', float_format='%.0f')
    content = content.encode(encoding='utf-8')
    outfile = used_args['outfile']
    with open(outfile, "wb") as fp:
        fp.write(content)
    pass


def do_something(used_args):
    # 统计每个LEVEL下的信息
    levels = "13,7,6,5,4,3,2,1"
    used_args['levels'] = levels
    stat_all_level_info(used_args)

    pass


def usage():
    print("======="*10)
    print("Usage:")
    print("1) python quantile_count.py -m heAuto_level_matrix/ -o outfile")
    print("2) python quantile_count.py --help")
    print("======="*10)
    exit()



def main():
    # init
    short_opts = 'hm:o:'
    long_opts = ['help', 'matrix_dir=', 'outfile=']
    args_names = [None, 'matrix_dir', 'outfile']

    # 生成required_options与opts_2_names
    ss = short_opts.replace(':','')
    required_options = []
    opts_2_names = {}
    for i in range(len(args_names)):
        if args_names[i] is None:
            continue
        else:
            short_o = '-' + ss[i]
            long_o = '--' + long_opts[i].replace('=','')
            required_options.append([short_o,long_o])
            opts_2_names[short_o] = args_names[i]
            opts_2_names[long_o] = args_names[i]

    # getopt
    options_dict = {}
    used_args = {}
    options, args = getopt.getopt(sys.argv[1:], short_opts, long_opts)
    for name, value in options:
        if name not in options_dict.keys():
            options_dict[name] = value
        if name in opts_2_names.keys():
            used_args[opts_2_names[name]] = value

    if '-h' in options_dict.keys() or '--help' in options_dict.keys():
        usage()
    print(used_args)

    # 检查必需提供的参数是否被提供，没有就提示一下，并打印使用说明
    for short_o, long_o in required_options:
        if short_o not in options_dict.keys() and long_o not in options_dict.keys():
            print(f"{short_o} or {long_o} must exist")
            usage()

    do_something(used_args)



if __name__ == "__main__":
    main()








