#!/usr/bin/perl -w
use strict ;

if (@ARGV < 3){
    print "Usage: perl $0 infile selectfile outfile\n" ;
    exit(1);
}

my ($infile, $selectfile, $outfile) = @ARGV ;

open (IN, $selectfile) || die "$selectfile, $!\n" ;
my @ids = ();
while(<IN>){
    chomp ;
    next if (m/^\#|^\s*$/);
    my ($id) = split ;
    $ids[$id] = 1 ;
}
close(IN);

if ($infile =~ /\.fa$|\.fasta$/i){
    open (IN, $infile) || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    $/ = ">" ;
    my $r_index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$/);
        if (!defined $ids[$r_index]){
            $r_index++ ;
            next ;
        }
        my $r_id = $r_index ;
        $r_index++ ;
        my ($id, $seq) = split /\n/, $_, 2 ;
        $seq =~ s/\n//g ;
        print OUT ">$r_id\n$seq\n" ;
    }
    $/ = "\n" ;
    close(IN);
    close(OUT);
}
elsif($infile =~ /\.fq$|\.fastq$/i){
    open (IN, $infile) || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my $r_index = 0 ;
    while(<IN>){
        my $id = $_ ;
        my $seq = <IN> ;
        my $id2 = <IN> ;
        my $qual = <IN> ;
        chomp($id, $seq, $id2, $qual);
        if (!defined $ids[$r_index]){
            $r_index++ ;
            next ;
        }
        my $r_id = $r_index ;
        $r_index++ ;
        print OUT "\@$r_id\n$seq\n\+\n$qual\n" ;
    }
    close(IN);
    close(OUT);
}
elsif($infile =~ /\.fq.gz$|\.fastq.gz$/i){
    open (IN, "gunzip -c $infile |") || die "$infile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    my $r_index = 0 ;
    while(<IN>){
        my $id = $_ ;
        my $seq = <IN> ;
        my $id2 = <IN> ;
        my $qual = <IN> ;
        chomp($id, $seq, $id2, $qual);
        if (!defined $ids[$r_index]){
            $r_index++ ;
            next ;
        }
        my $r_id = $r_index ;
        $r_index++ ;
        print OUT "\@$r_id\n$seq\n\+\n$qual\n" ;
    }
    close(IN);
    close(OUT);
}


