#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2022
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2022
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2022
my $ver="1.3";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","o=s","n=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{i}) || !defined($opts{o}) || !defined($opts{n}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:
		-i           .umi_gene.tsv file                   <infile>     must be given
		-o           stat result file                     <outfile>    must be given
		-n           total number                         <int>        must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $umifile  = $opts{i} ;
my $outfile = $opts{o} ;
my $total_reads = $opts{n} ;

## read umi gene file
&get_umi_gene_and_stat($umifile, $outfile, $total_reads);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_umi_gene_and_stat($umifile, $outfile);
sub get_umi_gene_and_stat()
{
    my ($infile, $outfile, $total_reads) = @_ ;

    open (IN, $infile) || die "$infile, $!\n" ;
    my $uniq = 0 ;
    my $total = 0 ;
    my @simu_arr = ();
    my $index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\#/);
        $index++ ;
        my ($bc, $umi, $gene, $num) = split ;
        $uniq++ ;
        $total += $num ;
        push @simu_arr, ($index)x$num ;
    }
    close(IN);
    #open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    #print OUT "UNIQ\t$uniq\n" ;
    #print OUT "Total\t$total\n" ;
    #print OUT "Sequencing Saturation\t", int((1-$uniq/$total)*10000+0.5)/100, "%\n" ;
    #close(OUT);

    &shuffle(\@simu_arr);

    my $percent = 0 ;
    my @uniqs = ();
    my $simu_uniq = 0 ;
    my %hstat = ();
    for (my $i=0; $i<@simu_arr; $i++){
        if (!defined $uniqs[$simu_arr[$i]]){
            $uniqs[$simu_arr[$i]] = 1 ;
            $simu_uniq++ ;
        }
        if ($i+1 >= $total*$percent/100){
            my $uniq = $simu_uniq ;
            my $simu_total = $i + 1 ;
            $hstat{$percent} = [$uniq, $simu_total, int((1-$uniq/$simu_total)*10000+0.5)/100] ;
            $percent += 5 ;
        }
    }

    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    print OUT "#Percent(%)\tUniq\tTotal\tSeqSaturateion(%)\n" ;
    $total_reads ||= $total ;
    for my $perc (sort {$a<=>$b} keys %hstat){
        my ($simu_uniq, $simu_total, $simu_saturat) = @{$hstat{$perc}} ;
        my $tr_uniq = int($simu_uniq/$total*$total_reads) ;
        my $tr_total = int($simu_total/$total*$total_reads) ;
        print OUT "$perc\t$tr_uniq\t$tr_total\t$simu_saturat\n" ;
    }
    close(OUT);

    return ;
}

#&shuffle(\@simu_arr);
sub shuffle()
{
    my ($asimu) = @_ ;

    my $num = scalar(@$asimu);
    for (my $i=0; $i<@$asimu; $i++){
        my $index = int(rand($num));
        ($asimu->[$i], $asimu->[$index]) = ($asimu->[$index], $asimu->[$i]);
    }

    return ;
}

