#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2022
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2022
my $ver="4.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);
use threads;
use threads::shared;

#####################

my %opts;
GetOptions(\%opts,"u=s","g=s","f=s","o=s","b=s","t=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{u}) || !defined($opts{g}) || !defined($opts{f}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver
		   v1.3: memory optimization.
		   v1.4: memory optimization.
		         delete -b bamfile option.
		   v3.0: memory/speed optimization.
		         change input file.
		   v3.1: change input file to map2gene
		   v3.2: memory optimization.
		   v4.0: speed optimization.

	Usage:

		-u           bc_umi_read.tsv.id file               <infile>     must be given
		-g           .map2gene file                        <infile>     must be given
		-f           features.tsv file                     <infile>     must be given
		-o           result file                           <outfile>    must be given
		-b           barcode.tsv file                      [infile]     optional [Null]
		-t           threads number                        [int]        optional [8]
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $umifile = $opts{u} ;
my $genefile = $opts{g} ;
my $ftsfile = $opts{f} ;
my $outfile = $opts{o} ;
my $bcfile = defined $opts{b} ? $opts{b} : "Null" ;
my $threads = defined $opts{t} ? $opts{t} : 8 ;

## get features gene
my %hgene2idx = ();
my %hidx2gene = ();
&get_features_gene($ftsfile, \%hgene2idx, \%hidx2gene);

## get gene info
my @rd_gene = ();
&get_gene_info($genefile, \@rd_gene, \%hgene2idx) ; 
&show_log("get reads-gene info done.") ;

## split umifile
my @umifiles = &split_umifile($umifile, $outfile, $threads);

## get reads umi info and out put
my @midoutfiles = ();
if ($bcfile eq 'Null'){
    for (my $i=0; $i<@umifiles; $i++){
        my $s_umifile = $umifiles[$i] ;
        threads->new(\&get_umi_info_and_gene, $s_umifile, \@rd_gene, \%hidx2gene, "$outfile.$i");
        push @midoutfiles, "$outfile.$i" ;
    }
}
else{
    for (my $i=0; $i<@umifiles; $i++){
        my $s_umifile = $umifiles[$i] ;
        threads->new(\&get_umi_info_and_gene2, $s_umifile, $bcfile, \@rd_gene, \%hidx2gene, "$outfile.$i");
        push @midoutfiles, "$outfile.$i" ;
    }
}

for my $thread (threads->list(threads::all)){
    $thread->join();
}
unlink(@umifiles);

`rm $outfile` if (-e $outfile) ;
if ($bcfile eq 'Null'){
    foreach my $midfile (@midoutfiles){
        `cat $midfile >> $outfile` ;
        `rm $midfile`;
    }
}
else{
    `rm $outfile.filter` if (-e "$outfile.filter");
    foreach my $midfile (@midoutfiles){
        `cat $midfile >> $outfile` ;
        `cat $midfile.filter >> $outfile.filter` ;
        `rm $midfile`;
        `rm $midfile.filter`;
    }
}

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_features_gene($ftsfile, \%hgene2idx, \%hidx2gene);
sub get_features_gene()
{
    my ($ftsfile, $ahgene2idx, $ahidx2gene) = @_ ;

    open (IN, $ftsfile) || die "$ftsfile, $!\n" ;
    my $index = 0 ;
    while(<IN>){
        chomp ;
        my ($gene_id, $gene_name) = split ;
        $ahgene2idx->{$gene_id} = $index ;
        $ahidx2gene->{$index} = $gene_id ;
        $index++ ;
    }
    close(IN);

    return ;
}

#&get_gene_info($genefile, \@rd_gene, \%hgene2idx) ; 
sub get_gene_info()
{
    my ($genefile, $ard_gene, $ahgene2idx) = @_ ;

    open (IN, $genefile) || die "$genefile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$|^\#/);
        my ($id, $maptype, @infos)= split ;
        for (my $i=0; $i<@infos; $i+=3){
            my ($gene, $type, $maplen) = @infos[$i..($i+2)] ;
            my $index = $ahgene2idx->{$gene} ;
            if (!defined $ard_gene->[$id]){
                $ard_gene->[$id] = $index ;
            }
            else{
                $ard_gene->[$id] .= "-$index" ;
            }
        }
    }
    close(IN);

    return ;
}

#my @umifiles = &split_umifile($umifile, $outfile, $threads);
sub split_umifile()
{
    my ($umifile, $outfile, $threads) = @_ ;

    my @umifiles = ();
    my $lineinfo = `wc -l $umifile` ;
    my $line_num = (split /\s+/, $lineinfo)[0] ;
    my $split_num = int($line_num/$threads) + 1 ;

    open (IN, $umifile) || die "$umifile, $!\n" ;
    open (OUT, ">$outfile.tmpumi.0") || die "$outfile.tmpumi.0, $!\n" ;
    push @umifiles, "$outfile.tmpumi.0" ;
    my $num = 0 ;
    my $index = 0 ;
    while(<IN>){
        chomp ;
        $num++ ;
        print OUT "$_\n" ;
        if ($num == $split_num){
            close(OUT) ;
            $index++ ;
            open (OUT, ">$outfile.tmpumi.$index") || die "$outfile.tmpumi.$index, $!\n" ;
            $num = 0 ;
            push @umifiles, "$outfile.tmpumi.$index" ;
        }
    }
    close(IN);
    close(OUT);

    return(@umifiles);
}

#&get_umi_info_and_gene2($umifile, $bcfile, \@rd_gene, \%hidx2gene, $outfile);
sub get_umi_info_and_gene2()
{
    my ($umifile, $bcfile, $ard_gene, $ahidx2gene, $outfile) = @_ ;

    open (IN, $bcfile) || die "$bcfile, $!\n" ;
    my %hbc = ();
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($bc) = split ;
        $hbc{$bc} = 1 ;
    }
    close(IN);

    open (IN, $umifile) || die "$umifile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    open (FIT, ">$outfile.filter") || die "$outfile.filter, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($bc, $umi, @ids) = split ;
        my $gene = &get_bc_umi_gene($ard_gene, $ahidx2gene, \@ids);
        next if ($gene eq "NULL") ;
        my $num = scalar(@ids) ;
        if (defined $hbc{$bc}){
            print OUT "$bc\t$umi\t$gene\t$num\n" ;
        }
        else{
            print FIT "$bc\t$umi\t$gene\t$num\n" ;
        }
    }
    close(IN);
    close(OUT);
    close(FIT);

    return ;
}

#&get_umi_info_and_gene($umifile, \@rd_gene, \%hidx2gene, $outfile);
sub get_umi_info_and_gene()
{
    my ($umifile, $ard_gene, $ahidx2gene, $outfile) = @_ ;

    open (IN, $umifile) || die "$umifile, $!\n" ;
    open (OUT, ">$outfile") || die "$outfile, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($bc, $umi, @ids) = split ;
        my $gene = &get_bc_umi_gene($ard_gene, $ahidx2gene, \@ids);
        next if ($gene eq "NULL") ;
        my $num = scalar(@ids) ;
        print OUT "$bc\t$umi\t$gene\t$num\n" ;
    }
    close(IN);
    close(OUT);

    return ;
}

#my $gene = &get_bc_umi_gene($ard_gene, $ahidx2gene, \@ids);
sub get_bc_umi_gene()
{
    my ($ard_gene, $ahidx2gene, $aids) = @_ ;

    my %hstat = ();
    for my $id (@{$aids}){
        next if (!defined $ard_gene->[$id]) ;
        my %hgene = ();
        my @indexs = split /\-/, $ard_gene->[$id] ;
        for my $gene_index (@indexs){
            $hgene{$gene_index} = 1;
        }
        for my $gene_index (keys %hgene){
            $hstat{$gene_index} ++ ;
        }
    }
    #my @indexs = sort {$hstat{$b}<=>$hstat{$a} || $a cmp $b} keys %hstat ;
    my @indexs = sort {$hstat{$b}<=>$hstat{$a}} keys %hstat ;
    if (scalar @indexs > 0){
        my $gene = $ahidx2gene->{$indexs[0]} ;
        return($gene) ;
    }
    else{
        return("NULL") ;
    }
}

