#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2023
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2023
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2023
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"id=s","o=s","e","h" );

#&help()if(defined $opts{h});
if(!defined($opts{id}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-id          reads mapping infofile dir            <indir>      must be given
		-o           result file                           <outfile>    must be given
		-e           only use exon(exclude intron)         [null]       optional     
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $indir = $opts{id} ;
my $outfile = $opts{o} ;
my @infiles = glob("$indir/*.mapinfo") ;

## get reads mapping info
my @genes = ();
my @genes_mt = ();
my @maps_id = ();
my $read_gene_fh ;
open ($read_gene_fh, ">$outfile.map2gene") || die "$outfile.map2gene, $!\n" ;
for my $file (@infiles){
    &get_reads_info($file, \@genes, \@genes_mt, \@maps_id, $read_gene_fh);
}
my %hstat = ();
for (my $i=0; $i<@maps_id; $i++){
    next if (!defined $maps_id[$i]) ;
    $hstat{$maps_id[$i]}++ ;
    next if (!defined $genes[$i]);
    print {$read_gene_fh} $i ;
    for my $gene (keys %{$genes[$i]}){
        my $maplen = $genes[$i]{$gene} ;
        print {$read_gene_fh} "\t$gene I $maplen" ;
    }
    print {$read_gene_fh} "\n" ;
}
close($read_gene_fh);

## out put stat file
&stat_and_output(\%hstat, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_reads_info($file, \@genes, \@genes_mt, \@maps_id, $read_gene_fh);
sub get_reads_info()
{
    my ($file, $agenes, $agenes_mt, $amaps_id, $read_gene_fh) = @_ ;

    open (IN, $file) || die "$file, $!\n" ;
    while(<IN>){
        chomp ;
        my ($id, $map_type, @infos) = split ;
        $amaps_id->[$id] ||= "MP" ;
        if ($infos[0] =~ m/Null/i){
            $amaps_id->[$id] = "CM" if ($map_type eq 'U');
            next ;
        }
        if ($map_type eq 'U'){
            print {$read_gene_fh} "$_\n" ;
            if (scalar @infos == 3){
                my ($gene_id, $type, $maplen) = @infos ;
                if ($type eq 'E'){
                    $amaps_id->[$id] = 'EX-T' ;
                }
                else{
                    $amaps_id->[$id] = 'IN-U' ;
                    if (!defined $opts{e}){
                        $amaps_id->[$id] = 'IN-T' ;
                    }
                }
            }
            else{
                my ($gene_id, $type, $maplen) = @infos[0..2] ;
                if ($type eq 'E'){
                    $amaps_id->[$id] = 'EX-M' ;
                }
                else{
                    $amaps_id->[$id] = 'IN-U' ;
                }
            }
        }
        else{
            my ($gene_id, $type, $maplen) = @infos[0..2] ;
            if ($type eq 'E'){
                print {$read_gene_fh} "$_\n" ;
                delete $agenes->[$id] ;
                if (defined $amaps_id->[$id] && ($amaps_id->[$id] eq 'AD-T')){
                    for (my $i=0; $i<@infos; $i+=3){
                        if (!defined $agenes_mt->[$id]{$infos[$i]}){
                            #$agenes_mt->[$id]{$infos[$i]}++ ;
                            $amaps_id->[$id] = "N-E" ;
                        }
                    }
                }
                elsif (defined $amaps_id->[$id] && ($amaps_id->[$id] eq 'N-E')){
                }
                else{
                    if (scalar(@infos) > 3){
                        $amaps_id->[$id] = "N-E" ;
                    }
                    else{
                        $amaps_id->[$id] = "AD-T" ;
                        for (my $i=0; $i<@infos; $i+=3){
                            $agenes_mt->[$id]{$infos[$i]}++ ; 
                        }
                    }
                }
            }
            else{
                next if (defined $amaps_id->[$id] && ($amaps_id->[$id] eq 'AD-T' || $amaps_id->[$id] eq 'N-E'));
                if (scalar(@infos) > 3){
                    $amaps_id->[$id] = 'N-I' ;
                    for (my $i=0; $i<@infos; $i+=3){
                        $agenes->[$id]{$infos[$i]} = $infos[$i+2] ;
                    }
                }
                else{
                    if (defined $amaps_id->[$id] && $amaps_id->[$id] eq 'ADIT'){
                        $amaps_id->[$id] = 'N-I' ;
                        $agenes->[$id]{$infos[0]} = $infos[2];
                    }
                    elsif (defined $amaps_id->[$id] && $amaps_id->[$id] eq 'N-I'){
                    }
                    else{
                        $amaps_id->[$id] = 'ADIT' ;
                        $agenes->[$id]{$infos[0]} = $infos[2] ;
                    }
                }
            }
        }
    }
    close(IN);

    return ;
}

#&stat_and_output(\%hstat, $outfile);
sub stat_and_output()
{
    my ($ahstat, $outfile) = @_ ;

    open (OUT, ">$outfile.stat") || die "$outfile.stat, $!\n" ;
    $ahstat->{'EX-T'} ||= 0 ;
    $ahstat->{'EX-M'} ||= 0 ;
    $ahstat->{'IN-T'} ||= 0 ;
    $ahstat->{'IN-U'} ||= 0 ;
    $ahstat->{'A-EX'} ||= 0 ;
    $ahstat->{'A-IN'} ||= 0 ;
    $ahstat->{'AD-T'} ||= 0 ;
    $ahstat->{'N-E'} ||= 0 ;
    $ahstat->{'ADIT'} ||= 0 ;
    $ahstat->{'N-I'} ||= 0 ;
    $ahstat->{'MP'} ||= 0 ;
    $ahstat->{'CM'} ||= 0 ;
    my $exon = $ahstat->{'EX-M'} + $ahstat->{'EX-T'} + $ahstat->{'A-EX'} + $ahstat->{'AD-T'} ;
    my $intron = $ahstat->{'IN-T'} + $ahstat->{'IN-U'} + $ahstat->{'A-IN'} + $ahstat->{'ADIT'} ;
    my $inter = $ahstat->{'CM'} ;
    my $transcript = $ahstat->{'EX-T'} + $ahstat->{'IN-T'} + $ahstat->{'AD-T'} ;
    if (!defined $opts{e}){
        $transcript += $ahstat->{'ADIT'} ;
    }
    my $antisense = $ahstat->{'A-EX'} + $ahstat->{'A-IN'} ;
    my $confmap = $exon + $intron + $inter ;
    my $map = $confmap + $ahstat->{'MP'} + $ahstat->{'N-E'} + $ahstat->{'N-I'} ;
    print OUT "Reads Mapped to Genome\t$map\n" ;
    print OUT "Reads Mapped Confidently to Genome\t$confmap\n" ;
    print OUT "Reads Mapped Confidently to Intergenic Regions\t$inter\n" ;
    print OUT "Reads Mapped Confidently to Intronic Regions\t$intron\n" ;
    print OUT "Reads Mapped Confidently to Exonic Regions\t$exon\n" ;
    print OUT "Reads Mapped Confidently to Transcriptome\t$transcript\n" ;
    print OUT "Reads Mapped Antisense to Gene\t$antisense\n" ;
    close(OUT);

    return ;
}

