#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"m=s","f=s","g=s","o=s","h" );

#&help()if(defined $opts{h});
if(!defined($opts{m}) || !defined($opts{f}) || !defined($opts{g}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-m           matrix.tsv file              <infile>     must be given
		-f           features.tsv file            <infile>     must be given
		-g           gff file                     <infile>     must be given
		-o           result file                  <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $mtfile = $opts{m} ;
my $ftfile = $opts{f} ;
#my $bcfile = $opts{b} ;
my $gfffile = $opts{g} ;
my $outfile = $opts{o} ;

## get gene length
my %hgenelen = ();
&get_gene_length($gfffile, \%hgenelen, $outfile);

## get gene index
my %hgeneindex = ();
&get_gene_index($ftfile, \%hgeneindex);

## get bc index
#my %hbcindex = ();
#&get_bc_index($bcfile, \%hbcindex);

## get cap gene number
my %hgenenum = ();
&get_cap_gene_num($mtfile, \%hgenenum);

## stat gene dist
&stat_gene_dist(\%hgenelen, \%hgeneindex, \%hgenenum, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_gene_length($gfffile, \%hgenelen, $outfile);
sub get_gene_length()
{
    my ($gfffile, $ahgenelen, $outfile) = @_ ;

    open (IN, $gfffile) || die "$gfffile, $!\n" ;
    open (OUT, ">$outfile.genelen") || die "$outfile.genelen, $!\n" ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($chr, undef, $type, $star, $end, undef, $ori, undef, $info) = split ;
        next if ($type !~ m/gene/);
        my $len = $end - $star + 1 ;
        if ($info =~ m/ID=(.*?)\;/){
            my $id = $1 ;
            $id =~ s/^gene[\-\:\_]// ;
            print OUT "$id\t$len\n" ;
            $ahgenelen->{$id} = $len ;
        }
    }
    close(IN) ;
    close(OUT) ;

    return ;
}

#&get_gene_index($ftfile, \%hgeneindex);
sub get_gene_index()
{
    my ($ftfile, $ahgeneindex) = @_ ;

    open (IN, $ftfile) || die "$ftfile, $!\n" ;
    my $index = 0 ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        $index++ ;
        my ($id, $name) = split ;
        $ahgeneindex->{$index} = $id ;
    }
    close(IN);

    return ;
}

#&get_bc_index($bcfile, \%hbcindex);
sub get_bc_index()
{
    my ($bcfile, $ahbcindex) = @_ ;


    return ;
}

#&get_cap_gene_num($mtfile, \%hgenenum);
sub get_cap_gene_num()
{
    my ($mtfile, $ahgenenum) = @_ ;

    open (IN, $mtfile) || die "$mtfile, $!\n" ;
    <IN> ;
    <IN> ;
    my $info = <IN> ;
    my ($total_gene_num, $total_barcode_num, $total_umi_num) = split /\s+/, $info ;
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($geneindex, $bcindex, $umi_num) = split ;
        $ahgenenum->{$geneindex} += $umi_num ;
    }
    close(IN);

    return ;
}

#&stat_gene_dist(\%hgenelen, \%hgeneindex, \%hgenenum, $outfile);
sub stat_gene_dist()
{
    my ($ahgenelen, $ahgeneindex, $ahgenenum, $outfile) = @_ ;

    my $dist = 100 ;
    my %hlen_dist = ();
    my %hcap_gene_dist = ();
    my $max_index = 0 ;
    open (OUT, ">$outfile.capgenenum") || die "$outfile.capgenenum, $!\n" ;
    print OUT "#Gene\tLength\tUMInum\n" ;
    for my $index (keys %{$ahgenenum}){
        my $id = $ahgeneindex->{$index} ;
        my $len = $ahgenelen->{$id} ;
        my $lenindex = int($len/$dist) ;
        my $num = $ahgenenum->{$index} ;
        $hlen_dist{$lenindex} += $num ;
        $hcap_gene_dist{$lenindex}++ ;
        print OUT "$id\t$len\t$num\n" ;
        if ($lenindex > $max_index){
            $max_index = $lenindex ;
        }
    }
    close(OUT);

    my %hgene_dist = ();
    for my $id (keys %{$ahgenelen}){
        my $len = $ahgenelen->{$id} ;
        my $index = int($len/$dist) ;
        $hgene_dist{$index}++ ;
        if ($index > $max_index){
            $max_index = $index ;
        }
    }

    open (OUT, ">$outfile.dist") || die "$outfile.dist, $!\n" ;
    print OUT "#MinLen-MaxLen\tGeneNum\tCapGeneNum\tUMInum\tUMI/CapGene\n" ;
    for (my $i=0; $i<=$max_index; $i++){
        my $gene_num = defined $hgene_dist{$i} ? $hgene_dist{$i} : 0 ;
        my $cap_gene_num = defined $hcap_gene_dist{$i} ? $hcap_gene_dist{$i} : 0 ;
        my $umi_num = defined $hlen_dist{$i} ? $hlen_dist{$i} : 0 ;
        my $ave = 0 ;
        if ($cap_gene_num != 0){
            $ave = int(($umi_num/$cap_gene_num)*100+0.5)/100 ;
        }
        my $min = $i*$dist ;
        my $max = ($i+1)*$dist ;
        print OUT "$min\-$max\t$gene_num\t$cap_gene_num\t$umi_num\t$ave\n" ;
    }
    close(OUT) ;

    return ;
}

